import logging
import struct
import enum
from io import BytesIO
import bitstruct
import binascii


# Connection flags.
CLEAN_START    = 0x02
WILL_FLAG      = 0x04
WILL_QOS_1     = 0x08
WILL_QOS_2     = 0x10
WILL_RETAIN    = 0x20
PASSWORD_FLAG  = 0x40
USER_NAME_FLAG = 0x80


def hexlify(data):
    if data is None:
        return None
    else:
        return binascii.hexlify(data).decode('ascii').upper()


# Control packet types.
class ControlPacketType(enum.IntEnum):
    CONNECT     = 1
    CONNACK     = 2
    PUBLISH     = 3
    PUBACK      = 4
    PUBREC      = 5
    PUBREL      = 6
    PUBCOMP     = 7
    SUBSCRIBE   = 8
    SUBACK      = 9
    UNSUBSCRIBE = 10
    UNSUBACK    = 11
    PINGREQ     = 12
    PINGRESP    = 13
    DISCONNECT  = 14
    AUTH        = 15


class ConnectReasonCode(enum.IntEnum):
    SUCCESS                              = 0
    V3_1_1_UNACCEPTABLE_PROTOCOL_VERSION = 1
    V3_1_1_IDENTIFIER_REJECTED           = 2
    V3_1_1_SERVER_UNAVAILABLE            = 3
    V3_1_1_BAD_USER_NAME_OR_PASSWORD     = 4
    V3_1_1_NOT_AUTHORIZED                = 5
    UNSPECIFIED_ERROR                    = 128
    MALFORMED_PACKET                     = 129
    PROTOCOL_ERROR                       = 130
    IMPLEMENTATION_SPECIFIC_ERROR        = 131
    UNSUPPORTED_PROTOCOL_VERSION         = 132
    CLIENT_IDENTIFIER_NOT_VALID          = 133
    BAD_USER_NAME_OR_PASSWORD            = 134
    NOT_AUTHORIZED                       = 135
    SERVER_UNAVAILABLE                   = 136
    SERVER_BUSY                          = 137
    BANNED                               = 138
    BAD_AUTHENTICATION_METHOD            = 140
    TOPIC_NAME_INVALID                   = 144
    PACKET_TOO_LARGE                     = 149
    QUOTA_EXCEEDED                       = 151
    PAYLOAD_FORMAT_INVALID               = 153
    RETAIN_NOT_SUPPORTED                 = 154
    QOS_NOT_SUPPORTED                    = 155
    USE_ANOTHER_SERVER                   = 156
    SERVER_MOVED                         = 157
    CONNECTION_RATE_EXCEEDED             = 159


class DisconnectReasonCode(enum.IntEnum):
    NORMAL_DISCONNECTION                   = 0
    DISCONNECT_WITH_WILL_MESSAGE           = 4
    UNSPECIFIED_ERROR                      = 128
    MALFORMED_PACKET                       = 129
    PROTOCOL_ERROR                         = 130
    IMPLEMENTATION_SPECIFIC_ERROR          = 131
    NOT_AUTHORIZED                         = 135
    SERVER_BUSY                            = 137
    SERVER_SHUTTING_DOWN                   = 139
    KEEP_ALIVE_TIMEOUT                     = 141
    SESSION_TAKEN_OVER                     = 142
    TOPIC_FILTER_INVALID                   = 143
    TOPIC_NAME_INVALID                     = 144
    RECEIVE_MAXIMUM_EXCEEDED               = 147
    TOPIC_ALIAS_INVALID                    = 148
    PACKET_TOO_LARGE                       = 149
    MESSAGE_RATE_TOO_HIGH                  = 150
    QUOTA_EXCEEDED                         = 151
    ADMINISTRATIVE_ACTION                  = 152
    PAYLOAD_FORMAT_INVALID                 = 153
    RETAIN_NOT_SUPPORTED                   = 154
    QOS_NOT_SUPPORTED                      = 155
    USE_ANOTHER_SERVER                     = 156
    SERVER_MOVED                           = 157
    SHARED_SUBSCRIPTIONS_NOT_SUPPORTED     = 158
    CONNECTION_RATE_EXCEEDED               = 159
    MAXIMUM_CONNECT_TIME                   = 160
    SUBSCRIPTION_IDENTIFIERS_NOT_SUPPORTED = 161
    WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED   = 162


class SubackReasonCode(enum.IntEnum):
    GRANTED_QOS_0                          = 0
    GRANTED_QOS_1                          = 1
    GRANTED_QOS_2                          = 2
    UNSPECIFIED_ERROR                      = 128
    IMPLEMENTATION_SPECIFIC_ERROR          = 131
    NOT_AUTHORIZED                         = 135
    TOPIC_FILTER_INVALID                   = 143
    PACKET_IDENTIFIER_IN_USE               = 145
    QUOTA_EXCEEDED                         = 151
    SHARED_SUBSCRIPTIONS_NOT_SUPPORTED     = 158
    SUBSCRIPTION_IDENTIFIERS_NOT_SUPPORTED = 161
    WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED   = 162


class UnsubackReasonCode(enum.IntEnum):
    SUCCESS                       = 0
    NO_SUBSCRIPTION_EXISTED       = 17
    UNSPECIFIED_ERROR             = 128
    IMPLEMENTATION_SPECIFIC_ERROR = 131
    NOT_AUTHORIZED                = 135
    TOPIC_FILTER_INVALID          = 143
    PACKET_IDENTIFIER_IN_USE      = 145


class PropertyIds(enum.IntEnum):
    PAYLOAD_FORMAT_INDICATOR          = 1
    MESSAGE_EXPIRY_INTERVAL           = 2
    CONTENT_TYPE                      = 3
    RESPONSE_TOPIC                    = 8
    CORRELATION_DATA                  = 9
    SUBSCRIPTION_IDENTIFIER           = 11
    SESSION_EXPIRY_INTERVAL           = 17
    ASSIGNED_CLIENT_IDENTIFIER        = 18
    SERVER_KEEP_ALIVE                 = 19
    AUTHENTICATION_METHOD             = 21
    AUTHENTICATION_DATA               = 22
    REQUEST_PROBLEM_INFORMATION       = 23
    WILL_DELAY_INTERVAL               = 24
    REQUEST_RESPONSE_INFORMATION      = 25
    RESPONSE_INFORMATION              = 26
    SERVER_REFERENCE                  = 28
    REASON_STRING                     = 31
    RECEIVE_MAXIMUM                   = 33
    TOPIC_ALIAS_MAXIMUM               = 34
    TOPIC_ALIAS                       = 35
    MAXIMUM_QOS                       = 36
    RETAIN_AVAILABLE                  = 37
    USER_PROPERTY                     = 38
    MAXIMUM_PACKET_SIZE               = 39
    WILDCARD_SUBSCRIPTION_AVAILABLE   = 40
    SUBSCRIPTION_IDENTIFIER_AVAILABLE = 41
    SHARED_SUBSCRIPTION_AVAILABLE     = 42


# MQTT 5.0
PROTOCOL_VERSION = 5

CF_FIXED_HEADER = bitstruct.compile('u4u4')

MAXIMUM_PACKET_SIZE = 268435455  # (128 ^ 4 - 1)

LOGGER = logging.getLogger(__name__)


class Error(Exception):
    pass


class MalformedPacketError(Error):
    pass


class TimeoutError(Error):
    pass


class PayloadReader(BytesIO):

    def __init__(self, data):
        super().__init__(data)
        self._length = len(data)

    def read(self, size):
        data = super().read(size)

        if len(data) != size:
            raise MalformedPacketError('Payload too short.')

        return data

    def read_all(self):
        return super().read()

    def is_data_available(self):
        return self.tell() < self._length


def pack_string(data):
    data = data.encode('utf-8')
    packed = struct.pack('>H', len(data))
    packed += data

    return packed


def unpack_string(payload):
    size = unpack_u16(payload)

    try:
        return payload.read(size).decode('utf-8')
    except UnicodeDecodeError:
        raise MalformedPacketError('String not UTF-8 encoded.')


def pack_u32(value):
    return struct.pack('>I', value)


def unpack_u32(payload):
    return struct.unpack('>I', payload.read(4))[0]


def pack_u16(value):
    return struct.pack('>H', value)


def unpack_u16(payload):
    return struct.unpack('>H', payload.read(2))[0]


def pack_u8(value):
    return struct.pack('B', value)


def unpack_u8(payload):
    return payload.read(1)[0]


def pack_property(property_id, value):
    return struct.pack('B', property_id) + {
        PropertyIds.PAYLOAD_FORMAT_INDICATOR: pack_u8,
        PropertyIds.MESSAGE_EXPIRY_INTERVAL: pack_u32,
        PropertyIds.CONTENT_TYPE: pack_string,
        PropertyIds.RESPONSE_TOPIC: pack_string,
        PropertyIds.CORRELATION_DATA: pack_binary,
        PropertyIds.SUBSCRIPTION_IDENTIFIER: pack_variable_integer,
        PropertyIds.SESSION_EXPIRY_INTERVAL: pack_u32,
        PropertyIds.ASSIGNED_CLIENT_IDENTIFIER: pack_string,
        PropertyIds.SERVER_KEEP_ALIVE: pack_u16,
        PropertyIds.AUTHENTICATION_METHOD: pack_string,
        PropertyIds.AUTHENTICATION_DATA: pack_binary,
        PropertyIds.REQUEST_PROBLEM_INFORMATION: pack_u8,
        PropertyIds.WILL_DELAY_INTERVAL: pack_u32,
        PropertyIds.REQUEST_RESPONSE_INFORMATION: pack_u8,
        PropertyIds.RESPONSE_INFORMATION: pack_string,
        PropertyIds.SERVER_REFERENCE: pack_string,
        PropertyIds.REASON_STRING: pack_string,
        PropertyIds.RECEIVE_MAXIMUM: pack_u16,
        PropertyIds.TOPIC_ALIAS_MAXIMUM: pack_u16,
        PropertyIds.TOPIC_ALIAS: pack_u16,
        PropertyIds.MAXIMUM_QOS: pack_u8,
        PropertyIds.RETAIN_AVAILABLE: pack_u8,
        PropertyIds.USER_PROPERTY: pack_string,
        PropertyIds.MAXIMUM_PACKET_SIZE: pack_u32,
        PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE: pack_u8,
        PropertyIds.SUBSCRIPTION_IDENTIFIER_AVAILABLE: pack_u8,
        PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE: pack_u8
    }[property_id](value)


def pack_properties(packet_name, properties):
    packed = b''

    for property_id, value in properties.items():
        packed += pack_property(property_id, value)

    return pack_variable_integer(len(packed)) + packed


def unpack_property(property_id, payload):
    return {
        PropertyIds.PAYLOAD_FORMAT_INDICATOR: unpack_u8,
        PropertyIds.MESSAGE_EXPIRY_INTERVAL: unpack_u32,
        PropertyIds.CONTENT_TYPE: unpack_string,
        PropertyIds.RESPONSE_TOPIC: unpack_string,
        PropertyIds.CORRELATION_DATA: unpack_binary,
        PropertyIds.SUBSCRIPTION_IDENTIFIER: unpack_variable_integer,
        PropertyIds.SESSION_EXPIRY_INTERVAL: unpack_u32,
        PropertyIds.ASSIGNED_CLIENT_IDENTIFIER: unpack_string,
        PropertyIds.SERVER_KEEP_ALIVE: unpack_u16,
        PropertyIds.AUTHENTICATION_METHOD: unpack_string,
        PropertyIds.AUTHENTICATION_DATA: unpack_binary,
        PropertyIds.REQUEST_PROBLEM_INFORMATION: unpack_u8,
        PropertyIds.WILL_DELAY_INTERVAL: unpack_u32,
        PropertyIds.REQUEST_RESPONSE_INFORMATION: unpack_u8,
        PropertyIds.RESPONSE_INFORMATION: unpack_string,
        PropertyIds.SERVER_REFERENCE: unpack_string,
        PropertyIds.REASON_STRING: unpack_string,
        PropertyIds.RECEIVE_MAXIMUM: unpack_u16,
        PropertyIds.TOPIC_ALIAS_MAXIMUM: unpack_u16,
        PropertyIds.TOPIC_ALIAS: unpack_u16,
        PropertyIds.MAXIMUM_QOS: unpack_u8,
        PropertyIds.RETAIN_AVAILABLE: unpack_u8,
        PropertyIds.USER_PROPERTY: unpack_string,
        PropertyIds.MAXIMUM_PACKET_SIZE: unpack_u32,
        PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE: unpack_u8,
        PropertyIds.SUBSCRIPTION_IDENTIFIER_AVAILABLE: unpack_u8,
        PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE: unpack_u8
    }[property_id](payload)


def unpack_properties(packet_name,
                      allowed_property_ids,
                      payload):
    """Return a dictionary of unpacked properties.

    """

    end_pos = unpack_variable_integer(payload)
    end_pos += payload.tell()
    properties = {}

    while payload.tell() < end_pos:
        property_id = payload.read(1)[0]

        if property_id not in allowed_property_ids:
            raise MalformedPacketError(
                f'Invalid property identifier {property_id}.')

        property_id = PropertyIds(property_id)
        properties[property_id] = unpack_property(property_id, payload)

    return properties


def pack_binary(data):
    packed = struct.pack('>H', len(data))
    packed += data

    return packed


def unpack_binary(payload):
    size = unpack_u16(payload)

    return payload.read(size)


def pack_variable_integer(value):
    if value == 0:
        packed = b'\x00'
    else:
        packed = b''

        while value > 0:
            encoded_byte = (value & 0x7f)
            value >>= 7

            if value > 0:
                encoded_byte |= 0x80

            packed += struct.pack('B', encoded_byte)

    return packed


def unpack_variable_integer(payload):
    value = 0
    multiplier = 1
    byte = 0x80

    while (byte & 0x80) == 0x80:
        byte = unpack_u8(payload)
        value += ((byte & 0x7f) * multiplier)
        multiplier <<= 7

    return value


def pack_fixed_header(message_type, flags, size):
    packed = CF_FIXED_HEADER.pack(message_type, flags)
    packed += pack_variable_integer(size)

    return packed


def unpack_fixed_header(payload):
    packet_type, flags = CF_FIXED_HEADER.unpack(payload)

    try:
        packet_type = ControlPacketType(packet_type)
    except ValueError:
        raise MalformedPacketError(f'Invalid packet type {packet_type}.')

    return packet_type, flags


def pack_connect(client_id,
                 clean_start,
                 will_topic,
                 will_message,
                 will_retain,
                 will_qos,
                 keep_alive_s,
                 properties):
    flags = 0

    if clean_start:
        flags |= CLEAN_START

    payload_length = len(client_id) + 2

    if will_topic:
        flags |= WILL_FLAG

        if will_retain:
            flags |= WILL_RETAIN

        if will_qos == 1:
            flags |= WILL_QOS_1
        elif will_qos == 2:
            flags |= WILL_QOS_2

        payload_length += 1
        packed_will_topic = pack_string(will_topic)
        payload_length += len(packed_will_topic)
        payload_length += len(will_message) + 2

    properties = pack_properties('CONNECT', properties)
    packed = pack_fixed_header(ControlPacketType.CONNECT,
                               0,
                               10 + payload_length + len(properties))
    packed += pack_string('MQTT')
    packed += struct.pack('>BBH',
                          PROTOCOL_VERSION,
                          flags,
                          keep_alive_s)
    packed += properties
    packed += pack_string(client_id)

    if flags & WILL_FLAG:
        packed += pack_variable_integer(0)
        packed += packed_will_topic
        packed += pack_binary(will_message)

    return packed


def unpack_connect(payload):
    if unpack_string(payload) != 'MQTT':
        raise MalformedPacketError('Invalid MQTT magic string.')

    if unpack_u8(payload) != PROTOCOL_VERSION:
        raise MalformedPacketError('Wrong protocol version.')

    flags = unpack_u8(payload)
    clean_start = bool(flags & CLEAN_START)
    keep_alive_s = unpack_u16(payload)
    properties = unpack_properties(
        'CONNECT',
        [
            PropertyIds.SESSION_EXPIRY_INTERVAL,
            PropertyIds.AUTHENTICATION_METHOD,
            PropertyIds.AUTHENTICATION_DATA,
            PropertyIds.REQUEST_PROBLEM_INFORMATION,
            PropertyIds.REQUEST_RESPONSE_INFORMATION,
            PropertyIds.RECEIVE_MAXIMUM,
            PropertyIds.TOPIC_ALIAS_MAXIMUM,
            PropertyIds.USER_PROPERTY,
            PropertyIds.MAXIMUM_PACKET_SIZE
        ],
        payload)
    client_id = unpack_string(payload)

    if flags & WILL_FLAG:
        will_properties = unpack_properties(
            'CONNECT-WILL',
            [
                PropertyIds.WILL_DELAY_INTERVAL,
                PropertyIds.PAYLOAD_FORMAT_INDICATOR,
                PropertyIds.MESSAGE_EXPIRY_INTERVAL,
                PropertyIds.CONTENT_TYPE,
                PropertyIds.RESPONSE_TOPIC,
                PropertyIds.USER_PROPERTY
            ],
            payload)
        will_topic = unpack_string(payload)
        will_message = unpack_binary(payload)
        will_retain = bool(flags & WILL_RETAIN)
    else:
        will_topic = None
        will_message = None
        will_retain = None

    if flags & USER_NAME_FLAG:
        user_name = unpack_string(payload)
    else:
        user_name = None

    if flags & PASSWORD_FLAG:
        password = unpack_binary(payload)
    else:
        password = None

    return (client_id,
            clean_start,
            will_topic,
            will_message,
            will_retain,
            keep_alive_s,
            properties,
            user_name,
            password)


def pack_connack(session_present,
                 reason,
                 properties):
    properties = pack_properties('CONNACK', properties)
    packed = pack_fixed_header(ControlPacketType.CONNACK,
                               0,
                               2 + len(properties))
    packed += pack_u8(int(session_present))
    packed += pack_u8(reason)
    packed += properties

    return packed


def unpack_connack(payload):
    flags = unpack_u8(payload)
    session_present = bool(flags & 1)

    try:
        reason = ConnectReasonCode(unpack_u8(payload))
    except ValueError:
        raise MalformedPacketError(f'Invalid CONNACK reason {reason}.')

    properties = unpack_properties(
        'CONNACK',
        [
            PropertyIds.SESSION_EXPIRY_INTERVAL,
            PropertyIds.ASSIGNED_CLIENT_IDENTIFIER,
            PropertyIds.SERVER_KEEP_ALIVE,
            PropertyIds.AUTHENTICATION_METHOD,
            PropertyIds.AUTHENTICATION_DATA,
            PropertyIds.RESPONSE_INFORMATION,
            PropertyIds.SERVER_REFERENCE,
            PropertyIds.REASON_STRING,
            PropertyIds.RECEIVE_MAXIMUM,
            PropertyIds.TOPIC_ALIAS_MAXIMUM,
            PropertyIds.MAXIMUM_QOS,
            PropertyIds.RETAIN_AVAILABLE,
            PropertyIds.USER_PROPERTY,
            PropertyIds.MAXIMUM_PACKET_SIZE,
            PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE,
            PropertyIds.SUBSCRIPTION_IDENTIFIER_AVAILABLE,
            PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE
        ],
        payload)

    return session_present, reason, properties


def pack_disconnect(reason):
    packed = pack_fixed_header(ControlPacketType.DISCONNECT, 0, 2)
    packed += struct.pack('B', reason)
    packed += pack_variable_integer(0)

    return packed


def unpack_disconnect(payload):
    if payload.is_data_available():
        reason = payload.read(1)[0]
    else:
        reason = 0

    try:
        reason = DisconnectReasonCode(reason)
    except ValueError:
        raise MalformedPacketError(f'Invalid DISCONNECT reason {reason}.')

    if payload.is_data_available():
        properties = unpack_properties(
            'DISCONNECT',
            [
                PropertyIds.SESSION_EXPIRY_INTERVAL,
                PropertyIds.SERVER_REFERENCE,
                PropertyIds.REASON_STRING,
                PropertyIds.USER_PROPERTY
            ],
            payload)
    else:
        properties = {}

    return reason, properties


def pack_subscribe(topic, packet_identifier):
    packed_topic = pack_string(topic)
    packed = pack_fixed_header(ControlPacketType.SUBSCRIBE,
                               2,
                               len(packed_topic) + 4)
    packed += struct.pack('>HB', packet_identifier, 0)
    packed += packed_topic
    packed += struct.pack('B', 0)

    return packed


def unpack_subscribe(payload):
    packet_identifier = unpack_u16(payload)
    properties = unpack_properties('SUBSCRIBE',
                                   [
                                       PropertyIds.SUBSCRIPTION_IDENTIFIER,
                                       PropertyIds.USER_PROPERTY
                                   ],
                                   payload)
    subscriptions = []

    while payload.is_data_available():
        topic = unpack_string(payload)
        options = unpack_u8(payload)
        subscriptions.append((topic, options))

    return packet_identifier, properties, subscriptions


def pack_suback(packet_identifier, reasons):
    properties = pack_properties('SUBACK', {})
    packed = pack_fixed_header(ControlPacketType.SUBACK,
                               0,
                               2 + len(properties) + len(reasons))
    packed += pack_u16(packet_identifier)
    packed += properties
    packed += reasons

    return packed


def unpack_suback(payload):
    packet_identifier = unpack_u16(payload)
    properties = unpack_properties('SUBACK',
                                   [
                                       PropertyIds.REASON_STRING,
                                       PropertyIds.USER_PROPERTY
                                   ],
                                   payload)
    reasons = []

    while payload.is_data_available():
        try:
            reason = SubackReasonCode(unpack_u8(payload))
        except ValueError:
            raise MalformedPacketError(f'Invalid SUBACK reason {reason}.')

        reasons.append(reason)

    return packet_identifier, properties, reasons


def pack_unsubscribe(topic, packet_identifier):
    packed_topic = pack_string(topic)
    packed = pack_fixed_header(ControlPacketType.UNSUBSCRIBE,
                               2,
                               len(packed_topic) + 3)
    packed += struct.pack('>HB', packet_identifier, 0)
    packed += packed_topic

    return packed


def unpack_unsubscribe(payload):
    packet_identifier = unpack_u16(payload)
    unpack_u8(payload)
    topics = []

    while payload.is_data_available():
        topics.append(unpack_string(payload))

    return packet_identifier, topics


def pack_unsuback(packet_identifier, reasons):
    packed = pack_fixed_header(ControlPacketType.UNSUBACK, 0, 3 + len(reasons))
    packed += pack_u16(packet_identifier)
    packed += pack_properties('UNSUBACK', {})
    packed += reasons

    return packed


def unpack_unsuback(payload):
    packet_identifier = unpack_u16(payload)
    properties = unpack_properties('UNSUBACK',
                                   [
                                       PropertyIds.REASON_STRING,
                                       PropertyIds.USER_PROPERTY
                                   ],
                                   payload)
    reasons = []

    while payload.is_data_available():
        try:
            reason = UnsubackReasonCode(unpack_u8(payload))
        except ValueError:
            raise MalformedPacketError(f'Invalid UNSUBACK reason {reason}.')

        reasons.append(reason)

    return packet_identifier, properties, reasons


def pack_publish(topic, message, retain, alias):
    flags = 0

    if retain:
        flags |= 1

    if alias is None:
        properties = b'\x00'
    else:
        properties = pack_properties('PUBLISH',
                                     {PropertyIds.TOPIC_ALIAS: alias})

    packed_topic = pack_string(topic)
    size = len(packed_topic) + len(message) + len(properties)
    packed = pack_fixed_header(ControlPacketType.PUBLISH, flags, size)
    packed += packed_topic
    packed += properties
    packed += message

    return packed


def unpack_publish(payload, qos):
    topic = unpack_string(payload)

    if qos > 0:
        raise MalformedPacketError('Only QoS 0 is supported.')

    properties = unpack_properties(
        'PUBLISH',
        [
            PropertyIds.PAYLOAD_FORMAT_INDICATOR,
            PropertyIds.MESSAGE_EXPIRY_INTERVAL,
            PropertyIds.CONTENT_TYPE,
            PropertyIds.RESPONSE_TOPIC,
            PropertyIds.CORRELATION_DATA,
            PropertyIds.SUBSCRIPTION_IDENTIFIER,
            PropertyIds.TOPIC_ALIAS,
            PropertyIds.USER_PROPERTY
        ],
        payload)

    message = payload.read_all()

    return topic, message, properties


def pack_pingreq():
    return pack_fixed_header(ControlPacketType.PINGREQ, 0, 0)


def pack_pingresp():
    return pack_fixed_header(ControlPacketType.PINGRESP, 0, 0)


def format_properties(properties):
    if not properties:
        return []

    lines = ['  Properties:']

    for identifier, value in properties.items():
        lines.append(f'    {identifier.name}({identifier.value}): {value}')

    return lines


def format_connect(payload):
    (client_id,
     clean_start,
     will_topic,
     will_message,
     will_retain,
     keep_alive_s,
     properties,
     user_name,
     password) = unpack_connect(payload)

    return [
        f'  ClientId:    {client_id}',
        f'  CleanStart:  {clean_start}',
        f'  WillTopic:   {will_topic}',
        f'  WillMessage: {hexlify(will_message)}',
        f'  WillRetain:  {will_retain}',
        f'  KeepAlive:   {keep_alive_s}',
        f'  UserName:    {user_name}',
        f'  Password:    {password}'
    ] + format_properties(properties)


def format_connack(payload):
    session_present, reason, properties = unpack_connack(payload)

    return [
        f'  SessionPresent: {session_present}',
        f'  Reason: {reason.name}({reason.value})'
    ] + format_properties(properties)


def format_publish(flags, payload):
    dup = bool((flags >> 3) & 0x1)
    qos = ((flags >> 1) & 0x3)
    retain = bool(flags & 0x1)
    topic, message, properties = unpack_publish(payload, qos)

    return [
        f'  DupFlag:    {dup}',
        f'  QoSLevel:   {qos}',
        f'  Retain:     {retain}',
        f'  Topic:      {topic}',
        f'  Message:    {hexlify(message)}',
        '  Properties:'
    ] + format_properties(properties)


def format_subscribe(payload):
    packet_identifier, properties, subscriptions = unpack_subscribe(payload)
    lines = [
        f'  PacketIdentifier: {packet_identifier}',
        '  Subscriptions:'
    ]

    for topic, flags in subscriptions:
        lines += [
            f'    Topic:             {topic}',
            f'    MaximumQoS:        {flags & 0x3}',
            f'    NoLocal:           {bool((flags >> 2) & 0x1)}',
            f'    RetainAsPublished: {bool((flags >> 3) & 0x1)}',
            f'    RetainHandling:    {(flags >> 4) & 0x3}'
        ]

    return lines


def format_suback(payload):
    packet_identifier, properties, reasons = unpack_suback(payload)

    return [
        f'  PacketIdentifier: {packet_identifier}',
        '  Properties:'
    ] + format_properties(properties) + [
        '  Reasons:'
    ] + [
        f'    {reason.name}({reason.value})' for reason in reasons
    ]


def format_unsubscribe(payload):
    packet_identifier, topics = unpack_unsubscribe(payload)

    return [
        f'  PacketIdentifier: {packet_identifier}',
        '  Topics:'
    ] + [f'    {topic}' for topic in topics]


def format_unsuback(payload):
    packet_identifier, properties, reasons = unpack_unsuback(payload)

    return [
        f'  PacketIdentifier: {packet_identifier}',
        '  Properties:'
    ] + format_properties(properties) + [
        '  Reasons:'
    ] + [
        f'    {reason.name}({reason.value})' for reason in reasons
    ]


def format_disconnect(payload):
    reason, properties = unpack_disconnect(payload)

    return [
        f'  Reason:     {reason.name}({reason.value})',
        '  Properties:'
    ] + format_properties(properties)


def format_packet(prefix, packet):
    lines = []

    try:
        packet_type, flags = unpack_fixed_header(packet)
        payload = PayloadReader(packet[1:])
        size = unpack_variable_integer(payload)
        packet_kind = packet_type.name
        lines.append(
            f'{prefix} {packet_kind}({packet_type.value}) packet of {len(packet)} '
            f'byte(s)')

        if packet_kind == 'CONNECT':
            lines += format_connect(payload)
        elif packet_kind == 'CONNACK':
            lines += format_connack(payload)
        elif packet_kind == 'PUBLISH':
            lines += format_publish(flags, payload)
        elif packet_kind == 'SUBSCRIBE':
            lines += format_subscribe(payload)
        elif packet_kind == 'SUBACK':
            lines += format_suback(payload)
        elif packet_kind == 'UNSUBSCRIBE':
            lines += format_unsubscribe(payload)
        elif packet_kind == 'UNSUBACK':
            lines += format_unsuback(payload)
        elif packet_kind == 'DISCONNECT':
            lines += format_disconnect(payload)
    except Exception as e:
        lines.append(f'  *** Malformed packet ({e}) ***')

    return lines


def format_connect_compact(payload):
    (client_id,
     _,
     will_topic,
     will_message,
     _,
     keep_alive_s,
     _,
     __name,
     _) = unpack_connect(payload)

    parts = [f'ClientId={client_id}']

    if will_topic is not None:
        parts.append(f'WillTopic={will_topic}')

    if will_message is not None:
        parts.append(f'WillMessage={hexlify(will_message)}')

    parts.append(f'KeepAlive={keep_alive_s}')

    return parts


def format_connack_compact(payload):
    _, reason, _ = unpack_connack(payload)

    return [f'Reason={reason.name}({reason.value})']


def format_publish_compact(flags, payload):
    qos = ((flags >> 1) & 0x3)
    topic, message, _ = unpack_publish(payload, qos)

    return [
        f'Topic={topic}',
        f'Message={hexlify(message)}',
    ]


def format_subscribe_compact(payload):
    _, _, subscriptions = unpack_subscribe(payload)
    parts = []

    for topic, _ in subscriptions:
        parts.append(f'Topic={topic}')

    return parts


def format_suback_compact(payload):
    _, _, reasons = unpack_suback(payload)
    parts = []

    for reason in reasons:
        parts.append(f'Reason={reason.name}({reason.value})')

    return parts


def format_unsubscribe_compact(payload):
    _, topics = unpack_unsubscribe(payload)
    parts = []

    for topic in topics:
        parts.append(f'Topic={topic}')

    return parts


def format_unsuback_compact(payload):
    _, _, reasons = unpack_unsuback(payload)
    parts = []

    for reason in reasons:
        parts.append(f'Reason={reason.name}({reason.value})')

    return parts


def format_disconnect_compact(payload):
    reason, _ = unpack_disconnect(payload)

    return [f'Reason={reason.name}({reason.value})']


def format_packet_compact(prefix, packet):
    try:
        packet_type, flags = unpack_fixed_header(packet)
        payload = PayloadReader(packet[1:])
        size = unpack_variable_integer(payload)
        packet_kind = packet_type.name
    except Exception as e:
        return f'{prefix} *** Malformed packet ({e}) ***'

    try:
        if packet_kind == 'CONNECT':
            extra = format_connect_compact(payload)
        elif packet_kind == 'CONNACK':
            extra = format_connack_compact(payload)
        elif packet_kind == 'PUBLISH':
            extra = format_publish_compact(flags, payload)
        elif packet_kind == 'SUBSCRIBE':
            extra = format_subscribe_compact(payload)
        elif packet_kind == 'SUBACK':
            extra = format_suback_compact(payload)
        elif packet_kind == 'UNSUBSCRIBE':
            extra = format_unsubscribe_compact(payload)
        elif packet_kind == 'UNSUBACK':
            extra = format_unsuback_compact(payload)
        elif packet_kind == 'DISCONNECT':
            extra = format_disconnect_compact(payload)
        else:
            extra = []

        extra = ', '.join(extra)

        if extra:
            extra = ': ' + extra
    except Exception as e:
        extra = f': *** Malformed packet ({e}) ***'

    return f'{prefix} {packet_kind}({packet_type.value}){extra}'



import unittest

import mqttools


class FormatPacketTest(unittest.TestCase):

    maxDiff = None

    def test_format_packet(self):
        datas = [
            (
                b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo',
                [
                    'Received CONNECT(1) packet of 18 byte(s)',
                    '  ClientId:    goo',
                    '  CleanStart:  True',
                    '  WillTopic:   None',
                    '  WillMessage: None',
                    '  WillRetain:  None',
                    '  KeepAlive:   1',
                    '  UserName:    None',
                    '  Password:    None'
                ],
                'Received CONNECT(1): ClientId=goo, KeepAlive=1'
            ),
            (
                b'\x10\x1a\x00\x04MQTT\x05\x06\x00\x00\x00\x00\x02id\x00\x00'
                b'\x03foo\x00\x03bar',
                [
                    'Received CONNECT(1) packet of 28 byte(s)',
                    '  ClientId:    id',
                    '  CleanStart:  True',
                    '  WillTopic:   foo',
                    '  WillMessage: 626172',
                    '  WillRetain:  False',
                    '  KeepAlive:   0',
                    '  UserName:    None',
                    '  Password:    None'
                ],
                'Received CONNECT(1): ClientId=id, WillTopic=foo, '
                'WillMessage=626172, KeepAlive=0'
            ),
            (
                b'\x20\x03\x00\x00\x00',
                [
                    'Received CONNACK(2) packet of 5 byte(s)',
                    '  SessionPresent: False',
                    '  Reason: SUCCESS(0)'
                ],
                'Received CONNACK(2): Reason=SUCCESS(0)'
            ),
            (
                b'\xc0\x00',
                [
                    'Received PINGREQ(12) packet of 2 byte(s)'
                ],
                'Received PINGREQ(12)'
            ),
            (
                b'\xd0\x00',
                [
                    'Received PINGRESP(13) packet of 2 byte(s)'
                ],
                'Received PINGRESP(13)'
            ),
            (
                b'\xe0\x0f\x80\x0d\x1f\x00\x0aSome error',
                [
                    'Received DISCONNECT(14) packet of 17 byte(s)',
                    '  Reason:     UNSPECIFIED_ERROR(128)',
                    '  Properties:',
                    '  Properties:',
                    '    REASON_STRING(31): Some error'
                ],
                'Received DISCONNECT(14): Reason=UNSPECIFIED_ERROR(128)'
            ),
            (
                b'\x30\x31\x00\x04\x2F\x61\x2F\x62\x00\x31\x32\x33\x34\x35'
                b'\x36\x37\x38\x39\x30\x30\x31\x32\x33\x34\x35\x36\x37\x38'
                b'\x39\x30\x30\x31\x32\x33\x34\x35\x36\x37\x38\x39\x30\x30'
                b'\x31\x32\x33\x34\x35\x36\x37\x38\x39',
                [
                    'Received PUBLISH(3) packet of 51 byte(s)',
                    '  DupFlag:    False',
                    '  QoSLevel:   0',
                    '  Retain:     False',
                    '  Topic:      /a/b',
                    '  Message:    313233343536373839303031323334353637383930'
                    '303132333435363738393030313233343536373839',
                    '  Properties:'
                ],
                'Received PUBLISH(3): Topic=/a/b, Message=31323334353637383930'
                '3031323334353637383930303132333435363738393030313233343536373'
                '839'
            ),
            (
                b'\x82\x0a\x00\x01\x00\x00\x04\x2f\x61\x2f\x62\x00',
                [
                    'Received SUBSCRIBE(8) packet of 12 byte(s)',
                    '  PacketIdentifier: 1',
                    '  Subscriptions:',
                    '    Topic:             /a/b',
                    '    MaximumQoS:        0',
                    '    NoLocal:           False',
                    '    RetainAsPublished: False',
                    '    RetainHandling:    0'
                ],
                'Received SUBSCRIBE(8): Topic=/a/b'
            ),
            (
                b'\x90\x04\x00\x01\x00\x00',
                [
                    'Received SUBACK(9) packet of 6 byte(s)',
                    '  PacketIdentifier: 1',
                    '  Properties:',
                    '  Reasons:',
                    '    GRANTED_QOS_0(0)'
                ],
                'Received SUBACK(9): Reason=GRANTED_QOS_0(0)'
            ),
            (
                b'\xa2\x0f\x00\x02\x00\x00\x04\x2f\x61\x2f\x62\x00\x04\x2f\x62'
                b'\x2f\x23',
                [
                    'Received UNSUBSCRIBE(10) packet of 17 byte(s)',
                    '  PacketIdentifier: 2',
                    '  Topics:',
                    '    /a/b',
                    '    /b/#'
                ],
                'Received UNSUBSCRIBE(10): Topic=/a/b, Topic=/b/#'
            ),
            (
                b'\xb0\x05\x00\x02\x00\x00\x00',
                [
                    'Received UNSUBACK(11) packet of 7 byte(s)',
                    '  PacketIdentifier: 2',
                    '  Properties:',
                    '  Reasons:',
                    '    SUCCESS(0)',
                    '    SUCCESS(0)'
                ],
                'Received UNSUBACK(11): Reason=SUCCESS(0), Reason=SUCCESS(0)'
            )
        ]

        for message, lines, compact_lines in datas:
            self.assertEqual(
                mqttools.common.format_packet('Received', message),
                lines)
            self.assertEqual(
                mqttools.common.format_packet_compact('Received', message),
                compact_lines)



import unittest

from mqttools.common import pack_connect
from mqttools.common import pack_u8
from mqttools.common import pack_u16
from mqttools.common import pack_u32
from mqttools.common import pack_binary
from mqttools.common import pack_string
from mqttools.common import pack_variable_integer
from mqttools.common import unpack_properties
from mqttools.common import PropertyIds
from mqttools.common import MalformedPacketError
from mqttools.common import PayloadReader


class PackUnpackTest(unittest.TestCase):

    def test_pack_connect(self):
        datas = [
            (
                ('client-id', True, '', b'', None, 0, 0, {}),
                b'\x10\x16\x00\x04MQTT\x05\x02\x00\x00\x00\x00\tclient-id'
            ),
            (
                ('abc', True, 'foo', b'bar', False, 0, 600, {}),
                b'\x10\x1b\x00\x04MQTT\x05\x06\x02X\x00\x00\x03abc\x00\x00\x03'
                b'foo\x00\x03bar'
            ),
            (
                ('abc', True, 'foo2', b'bar', True, 1, 3600, {}),
                b'\x10\x1c\x00\x04MQTT\x05\x2e\x0e\x10\x00\x00\x03abc\x00\x00'
                b'\x04foo2\x00\x03bar'
            )
        ]

        for args, expected_packed in datas:
            packed = pack_connect(*args)
            self.assertEqual(packed, expected_packed)

    def test_unpack_properties(self):
        buf = pack_u8(PropertyIds.PAYLOAD_FORMAT_INDICATOR)
        buf += pack_u8(5)
        buf += pack_u8(PropertyIds.MESSAGE_EXPIRY_INTERVAL)
        buf += pack_u32(6)
        buf += pack_u8(PropertyIds.CONTENT_TYPE)
        buf += pack_string('a')
        buf += pack_u8(PropertyIds.RESPONSE_TOPIC)
        buf += pack_string('b')
        buf += pack_u8(PropertyIds.CORRELATION_DATA)
        buf += pack_binary(b'c')
        buf += pack_u8(PropertyIds.SUBSCRIPTION_IDENTIFIER)
        buf += pack_variable_integer(7)
        buf += pack_u8(PropertyIds.SESSION_EXPIRY_INTERVAL)
        buf += pack_u32(8)
        buf += pack_u8(PropertyIds.ASSIGNED_CLIENT_IDENTIFIER)
        buf += pack_string('d')
        buf += pack_u8(PropertyIds.SERVER_KEEP_ALIVE)
        buf += pack_u16(9)
        buf += pack_u8(PropertyIds.AUTHENTICATION_METHOD)
        buf += pack_string('e')
        buf += pack_u8(PropertyIds.AUTHENTICATION_DATA)
        buf += pack_binary(b'f')
        buf += pack_u8(PropertyIds.REQUEST_PROBLEM_INFORMATION)
        buf += pack_u8(10)
        buf += pack_u8(PropertyIds.WILL_DELAY_INTERVAL)
        buf += pack_u32(11)
        buf += pack_u8(PropertyIds.REQUEST_RESPONSE_INFORMATION)
        buf += pack_u8(12)
        buf += pack_u8(PropertyIds.RESPONSE_INFORMATION)
        buf += pack_string('g')
        buf += pack_u8(PropertyIds.SERVER_REFERENCE)
        buf += pack_string('h')
        buf += pack_u8(PropertyIds.REASON_STRING)
        buf += pack_string('i')
        buf += pack_u8(PropertyIds.RECEIVE_MAXIMUM)
        buf += pack_u16(13)
        buf += pack_u8(PropertyIds.TOPIC_ALIAS_MAXIMUM)
        buf += pack_u16(14)
        buf += pack_u8(PropertyIds.TOPIC_ALIAS)
        buf += pack_u16(15)
        buf += pack_u8(PropertyIds.MAXIMUM_QOS)
        buf += pack_u8(16)
        buf += pack_u8(PropertyIds.RETAIN_AVAILABLE)
        buf += pack_u8(17)
        buf += pack_u8(PropertyIds.USER_PROPERTY)
        buf += pack_string('j')
        buf += pack_u8(PropertyIds.MAXIMUM_PACKET_SIZE)
        buf += pack_u32(18)
        buf += pack_u8(PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE)
        buf += pack_u8(19)
        buf += pack_u8(PropertyIds.SUBSCRIPTION_IDENTIFIER_AVAILABLE)
        buf += pack_u8(20)
        buf += pack_u8(PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE)
        buf += pack_u8(21)
        buf = pack_variable_integer(len(buf)) + buf

        properties = unpack_properties(
            'FOO',
            [
                PropertyIds.PAYLOAD_FORMAT_INDICATOR,
                PropertyIds.MESSAGE_EXPIRY_INTERVAL,
                PropertyIds.CONTENT_TYPE,
                PropertyIds.RESPONSE_TOPIC,
                PropertyIds.CORRELATION_DATA,
                PropertyIds.SUBSCRIPTION_IDENTIFIER,
                PropertyIds.SESSION_EXPIRY_INTERVAL,
                PropertyIds.ASSIGNED_CLIENT_IDENTIFIER,
                PropertyIds.SERVER_KEEP_ALIVE,
                PropertyIds.AUTHENTICATION_METHOD,
                PropertyIds.AUTHENTICATION_DATA,
                PropertyIds.REQUEST_PROBLEM_INFORMATION,
                PropertyIds.WILL_DELAY_INTERVAL,
                PropertyIds.REQUEST_RESPONSE_INFORMATION,
                PropertyIds.RESPONSE_INFORMATION,
                PropertyIds.SERVER_REFERENCE,
                PropertyIds.REASON_STRING,
                PropertyIds.RECEIVE_MAXIMUM,
                PropertyIds.TOPIC_ALIAS_MAXIMUM,
                PropertyIds.TOPIC_ALIAS,
                PropertyIds.MAXIMUM_QOS,
                PropertyIds.RETAIN_AVAILABLE,
                PropertyIds.USER_PROPERTY,
                PropertyIds.MAXIMUM_PACKET_SIZE,
                PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE,
                PropertyIds.SUBSCRIPTION_IDENTIFIER_AVAILABLE,
                PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE
            ],
            PayloadReader(buf))

        self.assertEqual(
            properties,
            {
                PropertyIds.PAYLOAD_FORMAT_INDICATOR: 5,
                PropertyIds.MESSAGE_EXPIRY_INTERVAL: 6,
                PropertyIds.CONTENT_TYPE: 'a',
                PropertyIds.RESPONSE_TOPIC: 'b',
                PropertyIds.CORRELATION_DATA: b'c',
                PropertyIds.SUBSCRIPTION_IDENTIFIER: 7,
                PropertyIds.SESSION_EXPIRY_INTERVAL: 8,
                PropertyIds.ASSIGNED_CLIENT_IDENTIFIER: 'd',
                PropertyIds.SERVER_KEEP_ALIVE: 9,
                PropertyIds.AUTHENTICATION_METHOD: 'e',
                PropertyIds.AUTHENTICATION_DATA: b'f',
                PropertyIds.REQUEST_PROBLEM_INFORMATION: 10,
                PropertyIds.WILL_DELAY_INTERVAL: 11,
                PropertyIds.REQUEST_RESPONSE_INFORMATION: 12,
                PropertyIds.RESPONSE_INFORMATION: 'g',
                PropertyIds.SERVER_REFERENCE: 'h',
                PropertyIds.REASON_STRING: 'i',
                PropertyIds.RECEIVE_MAXIMUM: 13,
                PropertyIds.TOPIC_ALIAS_MAXIMUM: 14,
                PropertyIds.TOPIC_ALIAS: 15,
                PropertyIds.MAXIMUM_QOS: 16,
                PropertyIds.RETAIN_AVAILABLE: 17,
                PropertyIds.USER_PROPERTY: 'j',
                PropertyIds.MAXIMUM_PACKET_SIZE: 18,
                PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE: 19,
                PropertyIds.SUBSCRIPTION_IDENTIFIER_AVAILABLE: 20,
                PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE: 21
            })

    def test_unpack_empty_properties(self):
        with self.assertRaises(MalformedPacketError):
            unpack_properties('FOO',
                              [
                                  PropertyIds.PAYLOAD_FORMAT_INDICATOR
                              ],
                              PayloadReader(b''))

    def test_unpack_invalid_property(self):
        with self.assertRaises(MalformedPacketError):
            unpack_properties('FOO',
                              [
                                  PropertyIds.PAYLOAD_FORMAT_INDICATOR
                              ],
                              PayloadReader(b'\x01\xff'))

    def test_unpack_invalid_property_for_packet(self):
        with self.assertRaises(MalformedPacketError):
            unpack_properties('FOO',
                              [
                                  PropertyIds.PAYLOAD_FORMAT_INDICATOR
                              ],
                              PayloadReader(b'\x01\x02\x00\x00\x00\x00'))


if __name__ == '__main__':
    unittest.main()



import logging
import sys
import argparse

from .version import __version__
from .client import Client
from .client import ConnectError
from .client import SessionResumeError
from .client import SubscribeError
from .client import UnsubscribeError
from .broker import Broker
from .broker import BrokerThread
from .common import MalformedPacketError
from .common import TimeoutError
from .common import SubackReasonCode
from .common import UnsubackReasonCode


def main():
    parser = argparse.ArgumentParser(description='MQTT Tools.')

    parser.add_argument('-d', '--debug', action='store_true')
    parser.add_argument('-l', '--log-level',
                        default='error',
                        choices=[
                            'debug', 'info', 'warning', 'error', 'critical'
                        ],
                        help='Set the logging level (default: %(default)s).')
    parser.add_argument('--version',
                        action='version',
                        version=__version__,
                        help='Print version information and exit.')

    # Workaround to make the subparser required in Python 3.
    subparsers = parser.add_subparsers(title='subcommands',
                                       dest='subcommand')
    subparsers.required = True

    # Import when used for less dependencies. For example, curses is
    # not part of all Python builds.
    from .subparsers import subscribe
    from .subparsers import publish
    from .subparsers import monitor
    from .subparsers import broker

    subscribe.add_subparser(subparsers)
    publish.add_subparser(subparsers)
    monitor.add_subparser(subparsers)
    broker.add_subparser(subparsers)

    args = parser.parse_args()

    level = logging.getLevelName(args.log_level.upper())
    logging.basicConfig(level=level, format='%(asctime)s %(message)s')

    if args.debug:
        args.func(args)
    else:
        try:
            args.func(args)
        except BaseException as e:
            sys.exit('error: ' + str(e))



import ssl
import asyncio
import time
import binascii
from humanfriendly import format_timespan

from ..client import Client
from ..common import Error
from . import to_int


def encode_message(message):
    try:
        return binascii.unhexlify(message)
    except ValueError:
        raise Error(f"Invalid hex string '{message}'.")


def create_message(message, size, number, fmt):
    if message is None:
        message_bytes = fmt.format(number).encode('ascii')
        extra = (size - len(message_bytes))

        if extra > 0:
            message_bytes += extra * b'\xa5'
        else:
            message_bytes = message_bytes[:size]
    else:
        message_bytes = encode_message(message)

    return message_bytes


async def publisher(host,
                    port,
                    client_id,
                    count,
                    size,
                    retain,
                    will_topic,
                    will_message,
                    will_retain,
                    session_expiry_interval,
                    cafile,
                    check_hostname,
                    topic,
                    message):
    if will_message is not None:
        will_message = encode_message(will_message)

    if cafile:
        print(f"CA File:  '{cafile}'")
        print(f"Check hostname: {check_hostname}")

        context = ssl.create_default_context(cafile=cafile)
        context.check_hostname = check_hostname
    else:
        context = None

    client = Client(host,
                    port,
                    client_id,
                    will_topic=will_topic,
                    will_message=will_message,
                    will_retain=will_retain,
                    session_expiry_interval=session_expiry_interval,
                    ssl=context)

    print(f"Connecting to '{host}:{port}'.")
    print()

    await client.start()

    fmt = '{{:0{}}}'.format(len(str(count - 1)))
    start_time = time.time()

    for number in range(count):
        message_bytes = create_message(message, size, number, fmt)
        client.publish(topic, message_bytes, retain=retain)

    elapsed_time = format_timespan(time.time() - start_time)
    print(f'Published {count} message(s) in {elapsed_time}.')

    await client.stop()


def _do_publish(args):
    asyncio.run(publisher(args.host,
                          args.port,
                          args.client_id,
                          args.count,
                          args.size,
                          args.retain,
                          args.will_topic,
                          args.will_message,
                          args.will_retain,
                          args.session_expiry_interval,
                          args.cafile,
                          not args.no_check_hostname,
                          args.topic,
                          args.message))


def add_subparser(subparsers):
    subparser = subparsers.add_parser('publish',
                                      description='Publish given topic.')
    subparser.add_argument('--host',
                           default='localhost',
                           help='Broker host (default: %(default)s).')
    subparser.add_argument('--port',
                           type=int,
                           default=1883,
                           help='Broker port (default: %(default)s).')
    subparser.add_argument('--client-id',
                           help='Client id (default: mqttools-<UUID[0..14]>).')
    subparser.add_argument(
        '--count',
        type=int,
        default=1,
        help='Number of times to publish the message (default: %(default)s).')
    subparser.add_argument(
        '--size',
        type=int,
        default=10,
        help='Generated message size (default: %(default)s).')
    subparser.add_argument('--retain',
                           action='store_true',
                           help='Retain the message.')
    subparser.add_argument('--will-topic', help='Will topic.')
    subparser.add_argument('--will-message', help='Will message.')
    subparser.add_argument('--will-retain',
                           action='store_true',
                           help='Retain the will message.')
    subparser.add_argument(
        '--session-expiry-interval',
        default=0,
        type=to_int,
        help='Session expiry interval in the range 0..0xffffffff (default: %(default)s).')
    subparser.add_argument(
        '--cafile',
        default='',
        help='CA file.')
    subparser.add_argument(
        '--no-check-hostname',
        action='store_true',
        help='Do not check certificate hostname.')
    subparser.add_argument('topic', help='Topic to publish.')
    subparser.add_argument(
        'message',
        nargs='?',
        help='Message to publish (default: <counter>\xa5...')
    subparser.set_defaults(func=_do_publish)



import logging
import asyncio
import unittest
import socket

import mqttools


BROKER_PORT = 10341


async def start_stop_clients():
    subscriber = mqttools.Client('localhost', BROKER_PORT)
    publisher = mqttools.Client('localhost', BROKER_PORT)

    await subscriber.start()
    await subscriber.subscribe('/apa')

    await publisher.start()
    publisher.publish('/apa', b'halloj')
    await publisher.stop()

    message = await subscriber.messages.get()

    if message != ('/apa', b'halloj'):
        raise Exception('Wrong message {}'.format(message))

    await subscriber.stop()


class BrokerThreadTest(unittest.TestCase):

    def connect(self):
        sock = socket.socket()

        try:
            sock.connect(('localhost', BROKER_PORT))
        finally:
            sock.close()

    def test_start_stop(self):
        broker = mqttools.BrokerThread(('localhost', BROKER_PORT))

        with self.assertRaises(ConnectionRefusedError):
            self.connect()

        broker.start()
        asyncio.run(start_stop_clients())
        self.connect()
        broker.stop()

        with self.assertRaises(mqttools.broker.NotRunningError):
            broker.stop()

        with self.assertRaises(ConnectionRefusedError):
            self.connect()



import ssl
import asyncio

from ..client import Client
from ..common import hexlify
from . import to_int


async def subscriber(host,
                     port,
                     cafile,
                     check_hostname,
                     client_id,
                     topic,
                     keep_alive_s,
                     session_expiry_interval):
    if cafile:
        print(f"CA File:  '{cafile}'")
        print(f"Check hostname: {check_hostname}")

        context = ssl.create_default_context(cafile=cafile)
        context.check_hostname = check_hostname
    else:
        context = None

    client = Client(host,
                    port,
                    client_id,
                    keep_alive_s=keep_alive_s,
                    session_expiry_interval=session_expiry_interval,
                    subscriptions=[topic],
                    topic_alias_maximum=10,
                    ssl=context)

    while True:
        print(f"Connecting to '{host}:{port}'.")
        await client.start()
        print('Connected.')

        while True:
            topic, message = await client.messages.get()

            if topic is None:
                print('Broker connection lost!')
                break

            print(f'Topic:   {topic}')
            print(f'Message: {hexlify(message)}')

        await client.stop()


def _do_subscribe(args):
    asyncio.run(subscriber(args.host,
                           args.port,
                           args.cafile,
                           not args.no_check_hostname,
                           args.client_id,
                           args.topic,
                           args.keep_alive,
                           args.session_expiry_interval))


def add_subparser(subparsers):
    subparser = subparsers.add_parser('subscribe',
                                      description='Subscribe for given topic.')
    subparser.add_argument('--host',
                           default='localhost',
                           help='Broker host (default: %(default)s).')
    subparser.add_argument('--port',
                           type=int,
                           default=1883,
                           help='Broker port (default: %(default)s).')
    subparser.add_argument('--client-id',
                           help='Client id (default: mqttools-<UUID[0..14]>).')
    subparser.add_argument('--keep-alive',
                           type=int,
                           default=0,
                           help=('Keep alive time in seconds (default: '
                                 '%(default)s). Give as 0 to disable keep '
                                 'alive.'))
    subparser.add_argument(
        '--session-expiry-interval',
        default=0,
        type=to_int,
        help='Session expiry interval in the range 0..0xffffffff (default: %(default)s).')
    subparser.add_argument(
        '--cafile',
        default='',
        help='CA file.')
    subparser.add_argument(
        '--no-check-hostname',
        action='store_true',
        help='Do not check certificate hostname.')
    subparser.add_argument('topic', help='Topic to subscribe for.')
    subparser.set_defaults(func=_do_subscribe)



import re
import asyncio
import logging
import threading
from collections import defaultdict

from .common import ControlPacketType
from .common import ConnectReasonCode
from .common import SubackReasonCode
from .common import UnsubackReasonCode
from .common import DisconnectReasonCode
from .common import PropertyIds
from .common import MalformedPacketError
from .common import PayloadReader
from .common import unpack_connect
from .common import pack_connack
from .common import pack_publish
from .common import unpack_publish
from .common import unpack_subscribe
from .common import pack_suback
from .common import unpack_unsubscribe
from .common import pack_unsuback
from .common import pack_pingresp
from .common import pack_disconnect
from .common import unpack_disconnect
from .common import format_packet
from .common import format_packet_compact
from .common import CF_FIXED_HEADER
from .common import MAXIMUM_PACKET_SIZE


LOGGER = logging.getLogger(__name__)


# ToDo: Session expiry. Keep alive.


class ConnectError(Exception):
    pass


class DisconnectError(Exception):
    pass


class ProtocolError(Exception):
    pass


class NotRunningError(Exception):
    pass


class Session(object):

    def __init__(self, client_id):
        self.client_id = client_id
        self.subscriptions = set()
        self.wildcard_subscriptions = set()
        self.expiry_interval = 0
        self.client = None
        self.maximum_packet_size = MAXIMUM_PACKET_SIZE
        self.will_topic = None
        self.will_message = None

    def clean(self):
        self.subscriptions = set()
        self.wildcard_subscriptions = set()
        self.expiry_interval = 0
        self.client = None
        self.maximum_packet_size = MAXIMUM_PACKET_SIZE
        self.will_topic = None
        self.will_message = None


def is_wildcards_in_topic(topic):
    return '#' in topic or '+' in topic


def compile_wildcards_topic(topic):
    pattern = topic.replace('+', '[^/]*')
    pattern = pattern.replace('/#', '.*')
    pattern = pattern.replace('#', '.*')
    pattern = '^' + pattern + '$'

    return re.compile(pattern)


class Client(object):

    def __init__(self, broker, reader, writer):
        self._broker = broker
        self._reader = reader
        self._writer = writer
        self._session = None
        self._disconnect_reason = DisconnectReasonCode.UNSPECIFIED_ERROR

    async def serve_forever(self):
        addr = self._writer.get_extra_info('peername')

        self.log_info('Serving client %s:%d.', *addr)

        try:
            packet_type, _, payload = await self.read_packet()

            if packet_type == ControlPacketType.CONNECT:
                self.on_connect(payload)
            else:
                raise ConnectError()

            await self.reader_loop()
        except ConnectError:
            pass
        except DisconnectError:
            self._disconnect_reason = DisconnectReasonCode.NORMAL_DISCONNECTION
        except asyncio.IncompleteReadError:
            self.log_debug('Client connection lost.')
        except Exception as e:
            self.log_debug('Reader task stopped by %r.', e)

            if isinstance(e, MalformedPacketError):
                self._disconnect_reason = DisconnectReasonCode.MALFORMED_PACKET
            elif isinstance(e, ProtocolError):
                self._disconnect_reason = DisconnectReasonCode.PROTOCOL_ERROR

            if self._session is not None:
                self.disconnect()

        if self._session is not None:
            self._session.client = None

            if self._session.will_topic is not None:
                if not self.is_normal_disconnection():
                    self._broker.publish(self._session.will_topic,
                                         self._session.will_message)

                    if self._session.will_retain:
                        self._broker.add_retained_message(self._session.will_topic,
                                                          self._session.will_message)

            if self._session.expiry_interval == 0:
                self._broker.remove_session(self._session.client_id)

        self.log_info('Closing client %r.', addr)

    async def reader_loop(self):
        while True:
            packet_type, flags, payload = await self.read_packet()

            if packet_type == ControlPacketType.PUBLISH:
                self.on_publish(payload, flags)
            elif packet_type == ControlPacketType.SUBSCRIBE:
                self.on_subscribe(payload)
            elif packet_type == ControlPacketType.UNSUBSCRIBE:
                self.on_unsubscribe(payload)
            elif packet_type == ControlPacketType.PINGREQ:
                self.on_pingreq()
            elif packet_type == ControlPacketType.DISCONNECT:
                self.on_disconnect(payload)
            else:
                raise ProtocolError()

    async def read_packet(self):
        buf = await self._reader.readexactly(1)
        packet_type, flags = CF_FIXED_HEADER.unpack(buf)
        size = 0
        multiplier = 1
        byte = 0x80

        while (byte & 0x80) == 0x80:
            buf += await self._reader.readexactly(1)
            byte = buf[-1]
            size += ((byte & 0x7f) * multiplier)
            multiplier <<= 7

        data = await self._reader.readexactly(size)

        if LOGGER.isEnabledFor(logging.DEBUG):
            for line in format_packet('Received', buf + data):
                self.log_debug(line)
        elif LOGGER.isEnabledFor(logging.INFO):
            self.log_info(format_packet_compact('Received', buf + data))

        return packet_type, flags, PayloadReader(data)

    def on_connect(self, payload):
        (client_id,
         clean_start,
         will_topic,
         will_message,
         will_retain,
         keep_alive_s,
         properties,
         user_name,
         password) = unpack_connect(payload)
        self._session, session_present = self._broker.get_session(
            client_id,
            clean_start)

        if session_present:
            self.log_info(
                'Session resumed with %d simple and %d wildcard '
                'subscriptions.',
                len(self._session.subscriptions),
                len(self._session.wildcard_subscriptions))

        self._session.client = self
        reason = ConnectReasonCode.SUCCESS

        if PropertyIds.AUTHENTICATION_METHOD in properties:
            reason = ConnectReasonCode.BAD_AUTHENTICATION_METHOD

        if PropertyIds.MAXIMUM_PACKET_SIZE in properties:
            maximum_packet_size = properties[PropertyIds.MAXIMUM_PACKET_SIZE]
            self._session.maximum_packet_size = maximum_packet_size

        self._session.will_topic = will_topic
        self._session.will_message = will_message
        self._session.will_retain = will_retain

        if PropertyIds.SESSION_EXPIRY_INTERVAL in properties:
            session_expiry_interval = properties[PropertyIds.SESSION_EXPIRY_INTERVAL]
            self._session.expiry_interval = session_expiry_interval

        if (user_name is not None) or (password is not None):
            reason = ConnectReasonCode.BAD_USER_NAME_OR_PASSWORD

        self._write_packet(pack_connack(
            session_present,
            reason,
            {
                PropertyIds.MAXIMUM_QOS: 0,
                PropertyIds.WILDCARD_SUBSCRIPTION_AVAILABLE: 0,
                PropertyIds.SHARED_SUBSCRIPTION_AVAILABLE: 0
            }))

        if reason != ConnectReasonCode.SUCCESS:
            raise ConnectError()

        self.log_info('Client connected.')

    def on_publish(self, payload, flags):
        topic, message, _ = unpack_publish(payload, (flags >> 1) & 3)

        if is_wildcards_in_topic(topic):
            raise MalformedPacketError(f'Invalid topic {topic} in publish.')

        if flags & 1:
            if message:
                self._broker.add_retained_message(topic, message)
            else:
                self._broker.remove_retained_message(topic)

        self._broker.publish(topic, message)

    def on_subscribe(self, payload):
        packet_identifier, _, subscriptions = unpack_subscribe(payload)
        reasons = bytearray()
        retained_messages = []

        for topic, _ in subscriptions:
            if is_wildcards_in_topic(topic):
                if topic not in self._session.wildcard_subscriptions:
                    self._session.wildcard_subscriptions.add(topic)
                    self._broker.add_wildcard_subscriber(topic, self._session)

                retained_messages += list(
                    self._broker.find_retained_messages_wildcards(topic))
            else:
                if topic not in self._session.subscriptions:
                    self._session.subscriptions.add(topic)
                    self._broker.add_subscriber(topic, self._session)

                retained_message = self._broker.find_retained_message(topic)

                if retained_message:
                    retained_messages.append(retained_message)

            reason = SubackReasonCode.GRANTED_QOS_0
            reasons.append(reason)

        self._write_packet(pack_suback(packet_identifier, reasons))

        for topic, message in retained_messages:
            self.publish(topic, message)

    def on_unsubscribe(self, payload):
        packet_identifier, topics = unpack_unsubscribe(payload)
        reasons = bytearray()

        for topic in topics:
            reason = UnsubackReasonCode.NO_SUBSCRIPTION_EXISTED

            if is_wildcards_in_topic(topic):
                if topic in self._session.wildcard_subscriptions:
                    self._session.wildcard_subscriptions.remove(topic)
                    self._broker.remove_wildcard_subscriber(topic, self._session)
                    reason = UnsubackReasonCode.SUCCESS
            elif topic in self._session.subscriptions:
                self._session.subscriptions.remove(topic)
                self._broker.remove_subscriber(topic, self._session)
                reason = UnsubackReasonCode.SUCCESS

            reasons.append(reason)

        self._write_packet(pack_unsuback(packet_identifier, reasons))

    def on_pingreq(self):
        self._write_packet(pack_pingresp())

    def on_disconnect(self, payload):
        unpack_disconnect(payload)

        raise DisconnectError()

    def publish(self, topic, message):
        self._write_packet(pack_publish(topic, message, False, None))

    def disconnect(self):
        self._write_packet(pack_disconnect(self._disconnect_reason))

    def _send_prefix(self, message):
        if len(message) <= self._session.maximum_packet_size:
            return 'Sending'
        else:
            return 'Not sending'

    def _write_packet(self, message):
        if LOGGER.isEnabledFor(logging.DEBUG):
            for line in format_packet(self._send_prefix(message), message):
                self.log_debug(line)
        elif LOGGER.isEnabledFor(logging.INFO):
            self.log_info(format_packet_compact(self._send_prefix(message),
                                                message))

        if len(message) <= self._session.maximum_packet_size:
            self._writer.write(message)

    def log_debug(self, fmt, *args):
        if LOGGER.isEnabledFor(logging.DEBUG):
            if self._session is None:
                LOGGER.debug(fmt, *args)
            else:
                LOGGER.debug(f'{self._session.client_id} {fmt}', *args)

    def log_info(self, fmt, *args):
        if LOGGER.isEnabledFor(logging.INFO):
            if self._session is None:
                LOGGER.info(fmt, *args)
            else:
                LOGGER.info(f'{self._session.client_id} {fmt}', *args)

    def is_normal_disconnection(self):
        return self._disconnect_reason == DisconnectReasonCode.NORMAL_DISCONNECTION


class Server:

    def __init__(self, serve_client, address):
        self.serve_client = serve_client

        if len(address) == 3:
            self._ssl = address[2]
        else:
            self._ssl = None

        self._host = address[0]
        self._port = address[1]
        self.server = None
        self.ready = asyncio.Event()

    async def serve_forever(self):
        try:
            self.server = await asyncio.start_server(self.serve_client,
                                                     self._host,
                                                     self._port,
                                                     ssl=self._ssl)
        except OSError as e:
            LOGGER.warning('%s', e)
            raise

        self.ready.set()
        server_address = self.server.sockets[0].getsockname()

        LOGGER.info('Listening for clients on %s:%d.', *server_address)

        async with self.server:
            await self.server.serve_forever()


class Broker(object):
    """A limited MQTT version 5.0 broker.

    `addresses` is a list of ``(host, port)`` and ``(host, port,
    ssl)`` tuples. It may also be the host string or one of the
    tuples. The broker will listen for clients on all given
    addresses. ``ssl`` is an SSL context passed to
    `asyncio.start_server()` as `ssl`.

    Create a broker and serve clients:

    >>> broker = Broker('localhost')
    >>> await broker.serve_forever()

    """

    def __init__(self, addresses):
        if isinstance(addresses, str):
            addresses = (addresses, 1883)

        if isinstance(addresses, tuple):
            addresses = [addresses]

        self._sessions = {}
        self._subscribers = defaultdict(list)
        self._wildcard_subscribers = []
        self._servers = []

        for address in addresses:
            self._servers.append(Server(self.serve_client, address))

        self._client_tasks = set()
        self._retained_messages = {}

    async def getsockname(self, index=0):
        server = self._servers[index]
        await server.ready.wait()

        return server.server.sockets[0].getsockname()

    async def serve_forever(self):
        """Setup a listener socket and forever serve clients. This coroutine
        only ends if cancelled by the user.

        """

        try:
            await asyncio.gather(
                *[server.serve_forever() for server in self._servers])
        except asyncio.CancelledError:
            # Cancel all client tasks as the TCP server leaves them
            # running.
            for client_task in self._client_tasks:
                client_task.cancel()

            self._client_tasks = set()
            raise

    async def serve_client(self, reader, writer):
        current_task = asyncio.current_task()
        self._client_tasks.add(current_task)
        client = Client(self, reader, writer)

        try:
            await client.serve_forever()
        finally:
            try:
                self._client_tasks.remove(current_task)
            except KeyError:
                pass

    def add_subscriber(self, topic, session):
        topic_sessions = self._subscribers[topic]

        if session not in topic_sessions:
            topic_sessions.append(session)

    def remove_subscriber(self, topic, session):
        topic_sessions = self._subscribers[topic]

        if session in topic_sessions:
            del topic_sessions[topic_sessions.index(session)]

    def add_wildcard_subscriber(self, topic, session):
        re_topic = compile_wildcards_topic(topic)
        self._wildcard_subscribers.append((topic, session, re_topic))

    def remove_wildcard_subscriber(self, topic, session):
        for index, subscriber in enumerate(self._wildcard_subscribers):
            if topic == subscriber[0] and session == subscriber[1]:
                del self._wildcard_subscribers[index]
                break

    def add_retained_message(self, topic, message):
        self._retained_messages[topic] = message

    def remove_retained_message(self, topic):
        try:
            del self._retained_messages[topic]
        except KeyError:
            pass

    def find_retained_messages_wildcards(self, topic):
        re_topic = compile_wildcards_topic(topic)

        for topic in self._retained_messages:
            mo = re_topic.match(topic)

            if mo:
                yield (topic, self._retained_messages[topic])

    def find_retained_message(self, topic):
        if topic in self._retained_messages:
            return (topic, self._retained_messages[topic])
        else:
            return None

    def iter_subscribers(self, topic):
        for session in self._subscribers[topic]:
            if session.client is not None:
                yield session

        for _, session, re_topic in self._wildcard_subscribers:
            if session.client is not None:
                mo = re_topic.match(topic)

                if mo:
                    yield session

    def get_session(self, client_id, clean_start):
        session_present = False

        if client_id in self._sessions:
            session = self._sessions[client_id]

            if clean_start:
                for topic in session.subscriptions:
                    self.remove_subscriber(topic, session)

                for topic in session.wildcard_subscriptions:
                    self.remove_wildcard_subscriber(topic, session)

                session.clean()
            else:
                session_present = True
        else:
            session = Session(client_id)
            self._sessions[client_id] = session

        return session, session_present

    def remove_session(self, client_id):
        del self._sessions[client_id]

    def  publish(self, topic, message):
        """Publish given topic and message to all subscribers.

        """

        for session in self.iter_subscribers(topic):
            session.client.publish(topic, message)


class BrokerThread(threading.Thread):
    """The same as :class:`Broker`, but running in a thread.

    Create a broker and serve clients for 60 seconds:

    >>> broker = BrokerThread('broker.hivemq.com')
    >>> broker.start()
    >>> time.sleep(60)
    >>> broker.stop()

    """

    def __init__(self, addresses):
        super().__init__()
        self._addresses = addresses
        self.daemon = True
        self._loop = asyncio.new_event_loop()
        self._broker_task = self._loop.create_task(self._run())
        self._running = False

    def run(self):
        asyncio.set_event_loop(self._loop)
        self._running = True

        try:
            self._loop.run_until_complete(self._broker_task)
        finally:
            self._loop.close()

    def stop(self):
        """Stop the broker. All clients will be disconnected and the thread
        will terminate.

        """

        if not self._running:
            raise NotRunningError('The broker is already stopped.')

        self._running = False

        def cancel_broker_task():
            self._broker_task.cancel()

        self._loop.call_soon_threadsafe(cancel_broker_task)
        self.join()

    async def _run(self):
        broker = Broker(self._addresses)

        try:
            await broker.serve_forever()
        except asyncio.CancelledError:
            pass



import logging
import asyncio
import unittest
import threading
import queue
import socket

import mqttools


HOST = 'localhost'
PORT = 0


class Client(threading.Thread):

    def __init__(self, host, port):
        super().__init__()
        self._host = host
        self._port = port
        self.messages = []
        self.done = threading.Event()

    async def client(self):
        client = mqttools.Client(self._host,
                                 self._port,
                                 'goo',
                                 response_timeout=1,
                                 topic_alias_maximum=0,
                                 keep_alive_s=1)

        for _ in range(6):
            await client.start()
            self.messages.append(await client.messages.get())
            await client.stop()

    def run(self):
        asyncio.run(self.client())
        self.done.set()


class ReconnectTest(unittest.TestCase):

    def test_reconnect(self):
        listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        listener.bind((HOST, PORT))
        listener.listen()

        client_thread = Client(*listener.getsockname())
        client_thread.daemon = True
        client_thread.start()

        client, _ = listener.accept()

        # CONNECT
        self.assertEqual(
            client.recv(18),
            b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo')
        # CONNACK
        self.assertEqual(client.send(b'\x20\x03\x00\x00\x00'), 5)
        # PINGREQ
        self.assertEqual(client.recv(2), b'\xc0\x00')
        # PINGRESP
        self.assertEqual(client.send(b'\xd0\x00'), 2)

        # Connection closed by the broker.
        client.close()

        # Wait for another connection.
        client, _ = listener.accept()

        # CONNECT
        self.assertEqual(
            client.recv(18),
            b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo')
        # CONNACK
        self.assertEqual(client.send(b'\x20\x03\x00\x00\x00'), 5)
        # PINGREQ
        self.assertIn(client.recv(2), [b'', b'\xc0\x00'])

        # Don't respond to the ping request and wait for another
        # connection.
        client2, _ = listener.accept()
        client.close()

        # CONNECT
        self.assertEqual(
            client2.recv(18),
            b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo')
        # CONNACK
        self.assertEqual(client2.send(b'\x20\x03\x00\x00\x00'), 5)

        client2.close()

        # Wait for another connection.
        client, _ = listener.accept()

        # CONNECT
        self.assertEqual(
            client.recv(18),
            b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo')
        # CONNACK
        self.assertEqual(client.send(b'\x20\x03\x00\x00\x00'), 5)
        # DISCONNECT
        self.assertEqual(
            client.send(b'\xe0\x0f\x80\x0d\x1f\x00\x0aSome error'),
            17)

        # Wait for another connection.
        client2, _ = listener.accept()
        client.close()

        # CONNECT
        self.assertEqual(
            client2.recv(18),
            b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo')
        # CONNACK
        self.assertEqual(client2.send(b'\x20\x03\x00\x00\x00'), 5)
        # PUBLISH malformed packet with non-UTF-8 topic.
        self.assertEqual(client2.send(b'\x30\x07\x00\x01\xff\x00apa'), 9)
        # DISCONNECT with reason malformed packet
        self.assertEqual(client2.recv(4), b'\xe0\x02\x81\x00')

        # Wait for another connection.
        client, _ = listener.accept()
        client2.close()

        # CONNECT
        self.assertEqual(
            client.recv(18),
            b'\x10\x10\x00\x04MQTT\x05\x02\x00\x01\x00\x00\x03goo')
        # CONNACK
        self.assertEqual(client.send(b'\x20\x03\x00\x00\x00'), 5)

        client.close()
        listener.close()
        client_thread.done.wait()
        self.assertEqual(client_thread.messages, 6 * [(None, None)])


logging.basicConfig(level=logging.DEBUG)


if __name__ == '__main__':
    unittest.main()



import ssl
import logging
import asyncio
import unittest

import mqttools


class BrokerTest(unittest.TestCase):

    def create_broker(self):
        broker = mqttools.Broker(('localhost', 0))

        async def broker_wrapper():
            with self.assertRaises(asyncio.CancelledError):
                await broker.serve_forever()

        return broker, asyncio.create_task(broker_wrapper())

    def test_multiple_subscribers(self):
        asyncio.run(self.multiple_subscribers())

    async def multiple_subscribers(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup subscriber 1.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup subscriber 2.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su2'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer_2.write(subscribe)
            suback = await reader_2.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_3, writer_3 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_3.write(connect)
            connack = await reader_3.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish a topic.
            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer_3.write(publish)

            # Receive the publish in both subscribers.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/b\x00apa')
            publish = await reader_2.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/b\x00apa')

            # Cleanly disconnect subscriber 1.
            disconnect = b'\xe0\x02\x00\x00'
            writer_1.write(disconnect)
            writer_1.close()

            # Publish another topic, now only to subscriber 2.
            publish = b'\x30\x0a\x00\x04/a/b\x00boo'
            writer_3.write(publish)

            # Receive the publish in subscriber 2.
            publish = await reader_2.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/b\x00boo')

            writer_2.close()
            writer_3.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_subscribe_to_same_topic_twice(self):
        asyncio.run(self.subscribe_to_same_topic_twice())

    async def subscribe_to_same_topic_twice(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Subscribe to "/a/b" and "/b/#" twice, all should be
            # successful. Subscribe to "/d/e" once.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            for _ in range(2):
                subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
                writer_1.write(subscribe)
                suback = await reader_1.readexactly(6)
                self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            for _ in range(2):
                subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/b/#\x00'
                writer_1.write(subscribe)
                suback = await reader_1.readexactly(6)
                self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/d/e\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish "/a/b", "/b/c" and "/d/e" once. They should all
            # be received once.
            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/b/c\x00cow'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/d/e\x00123'
            writer_2.write(publish)

            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/b\x00apa')
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/b/c\x00cow')
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/d/e\x00123')

            # Unsubscribe from "/a/b" and "/b/#" and publish
            # again. Only "/d/e" should be received.
            unsubscribe = b'\xa2\x0f\x00\x02\x00\x00\x04/a/b\x00\x04/b/#'
            writer_1.write(unsubscribe)
            unsuback = await reader_1.readexactly(7)
            self.assertEqual(unsuback, b'\xb0\x05\x00\x02\x00\x00\x00')

            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/b/c\x00cow'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/d/e\x00123'
            writer_2.write(publish)

            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/d/e\x00123')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_unsubscribe(self):
        asyncio.run(self.unsubscribe())

    async def unsubscribe(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber. Subscribe to /a/a, /a/b, /a/c and /a/d.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = (
                b'\x82\x1f\x00\x01\x00\x00\x04/a/a\x00\x00\x04/a/b\x00\x00\x04'
                b'/a/c\x00\x00\x04/a/d\x00')
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(9)
            self.assertEqual(suback, b'\x90\x07\x00\x01\x00\x00\x00\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish /a/a and /a/c.
            publish = b'\x30\x0a\x00\x04/a/a\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/a/c\x00cow'
            writer_2.write(publish)

            # Receive /a/a and /a/c in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/a\x00apa')
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/c\x00cow')

            # Unsubscribe from /a/a and /a/c.
            unsubscribe = b'\xa2\x0f\x00\x02\x00\x00\x04/a/a\x00\x04/a/c'
            writer_1.write(unsubscribe)
            unsuback = await reader_1.readexactly(7)
            self.assertEqual(unsuback, b'\xb0\x05\x00\x02\x00\x00\x00')

            # Publish /a/a, /a/c and then /a/b, /a/a should not be
            # received by the subscriber.
            publish = b'\x30\x0a\x00\x04/a/a\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/a/c\x00cow'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/a/b\x00mas'
            writer_2.write(publish)

            # Receive /a/b in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/b\x00mas')

            # Unsubscribe from /a/b.
            unsubscribe = b'\xa2\x09\x00\x02\x00\x00\x04/a/b'
            writer_1.write(unsubscribe)
            unsuback = await reader_1.readexactly(6)
            self.assertEqual(unsuback, b'\xb0\x04\x00\x02\x00\x00')

            # Unsubscribe from /a/b again should fail.
            unsubscribe = b'\xa2\x09\x00\x02\x00\x00\x04/a/b'
            writer_1.write(unsubscribe)
            unsuback = await reader_1.readexactly(6)
            self.assertEqual(unsuback, b'\xb0\x04\x00\x02\x00\x11')

            # Unsubscribe from /a/# is an invalid topic in the current
            # implementation.
            unsubscribe = b'\xa2\x09\x00\x02\x00\x00\x04/a/#'
            writer_1.write(unsubscribe)
            unsuback = await reader_1.readexactly(6)
            self.assertEqual(unsuback, b'\xb0\x04\x00\x02\x00\x11')

            # Publish /a/b and then /a/d, /a/b should not be received
            # by the subscriber.
            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/a/d\x00mas'
            writer_2.write(publish)

            # Receive /a/d in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/d\x00mas')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_multi_level_wildcard_tennis(self):
        asyncio.run(self.multi_level_wildcard_tennis())

    async def multi_level_wildcard_tennis(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber and subscribe to
            # "sport/tennis/player1/#".
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x1c\x00\x01\x00\x00\x16sport/tennis/player1/#\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish "sport/tennis/player1".
            publish = b'\x30\x1a\x00\x14sport/tennis/player1\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(28)
            self.assertEqual(
                publish,
                b'\x30\x1a\x00\x14sport/tennis/player1\x00apa')

            # Publish "sport/tennis/player1/ranking".
            publish = b'\x30\x22\x00\x1csport/tennis/player1/ranking\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(36)
            self.assertEqual(
                publish,
                b'\x30\x22\x00\x1csport/tennis/player1/ranking\x00apa')

            # Publish "sport/tennis/player1/ranking/wimbledon".
            publish = (
                b'\x30\x2c\x00\x26sport/tennis/player1/ranking/wimbledon\x00apa')
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(46)
            self.assertEqual(
                publish,
                b'\x30\x2c\x00\x26sport/tennis/player1/ranking/wimbledon\x00apa')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_multi_level_wildcard_any(self):
        asyncio.run(self.multi_level_wildcard_any())

    async def multi_level_wildcard_any(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber and subscribe to "#".
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x07\x00\x01\x00\x00\x01#\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish "sport/tennis/player1".
            publish = b'\x30\x1a\x00\x14sport/tennis/player1\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(28)
            self.assertEqual(
                publish,
                b'\x30\x1a\x00\x14sport/tennis/player1\x00apa')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_single_level_wildcard_only(self):
        asyncio.run(self.single_level_wildcard_only())

    async def single_level_wildcard_only(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber and subscribe to "+".
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x07\x00\x01\x00\x00\x01+\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish "sport".
            publish = b'\x30\x0b\x00\x05sport\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(13)
            self.assertEqual(
                publish,
                b'\x30\x0b\x00\x05sport\x00apa')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_single_level_wildcard_middle(self):
        asyncio.run(self.single_level_wildcard_middle())

    async def single_level_wildcard_middle(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber and subscribe to "sport/+/player1".
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x15\x00\x01\x00\x00\x0fsport/+/player1\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish "sport/tennis/player1".
            publish = b'\x30\x1a\x00\x14sport/tennis/player1\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(28)
            self.assertEqual(
                publish,
                b'\x30\x1a\x00\x14sport/tennis/player1\x00apa')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_single_level_wildcard_end(self):
        asyncio.run(self.single_level_wildcard_end())

    async def single_level_wildcard_end(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber and subscribe to "sport/tennis/+".
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x14\x00\x01\x00\x00\x0esport/tennis/+\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish "sport/tennis/player1/ranking". It should not be
            # received.
            publish = b'\x30\x22\x00\x1csport/tennis/player1/ranking\x00apa'
            writer_2.write(publish)

            # Publish "sport/tennis/player1".
            publish = b'\x30\x1a\x00\x14sport/tennis/player1\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(28)
            self.assertEqual(
                publish,
                b'\x30\x1a\x00\x14sport/tennis/player1\x00apa')

            # Publish "sport/tennis/player2".
            publish = b'\x30\x1a\x00\x14sport/tennis/player2\x00apa'
            writer_2.write(publish)

            # Receive the publish.
            publish = await reader_1.readexactly(28)
            self.assertEqual(
                publish,
                b'\x30\x1a\x00\x14sport/tennis/player2\x00apa')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_unsubscribe_wildcard(self):
        asyncio.run(self.unsubscribe_wildcard())

    async def unsubscribe_wildcard(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber. Subscribe to a/# and b/+.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = (
                b'\x82\x11\x00\x01\x00\x00\x04/a/#\x00\x00\x04/b/+\x00')
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(7)
            self.assertEqual(suback, b'\x90\x05\x00\x01\x00\x00\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish /a/a and /b/c.
            publish = b'\x30\x0a\x00\x04/a/a\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/b/c\x00cow'
            writer_2.write(publish)

            # Receive /a/a and /b/c in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/a\x00apa')
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/b/c\x00cow')

            # Unsubscribe from /a/#.
            unsubscribe = b'\xa2\x09\x00\x02\x00\x00\x04/a/#'
            writer_1.write(unsubscribe)
            unsuback = await reader_1.readexactly(6)
            self.assertEqual(unsuback, b'\xb0\x04\x00\x02\x00\x00')

            # Publish /a/a and then /b/c, /a/a should not be received
            # by the subscriber.
            publish = b'\x30\x0a\x00\x04/a/a\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/b/c\x00cow'
            writer_2.write(publish)

            # Receive /b/c in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/b/c\x00cow')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_resume_session(self):
        asyncio.run(self.resume_session())

    async def resume_session(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Try to resume a session that does not exist. A new
            # session should be created and immediately removed when
            # the connetion ends as the expiry interval is 0 (not
            # given).
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = (b'\x10\x10\x00\x04MQTT\x05\x00\x00\x00\x00\x00\x03res')
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')
            writer_1.close()

            # Try to resume a session that does not exist. A new
            # session should be created.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = (b'\x10\x15\x00\x04MQTT\x05\x00\x00\x00\x05\x11\xff\xff'
                       b'\xff\xff\x00\x03res')
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')
            writer_1.close()

            # Resume the session.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = (b'\x10\x15\x00\x04MQTT\x05\x00\x00\x00\x05\x11\xff\xff'
                       b'\xff\xff\x00\x03res')
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x01\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Setup a publisher.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish /a/b.
            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer_2.write(publish)

            # Receive /a/b in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/b\x00apa')

            # Perform a clean start and subscribe to /a/c.
            writer_1.close()
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03res'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/c\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Publish /a/b and then /a/c.
            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer_2.write(publish)
            publish = b'\x30\x0a\x00\x04/a/c\x00apa'
            writer_2.write(publish)

            # Receive /a/c in the subscriber.
            publish = await reader_1.readexactly(12)
            self.assertEqual(publish, b'\x30\x0a\x00\x04/a/c\x00apa')

            writer_1.close()
            writer_2.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_ping(self):
        asyncio.run(self.ping())

    async def ping(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the pinger.
            reader, writer = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer.write(connect)
            connack = await reader.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            pingreq = b'\xc0\x00'
            writer.write(pingreq)
            pingresp = await reader.readexactly(2)
            self.assertEqual(pingresp, b'\xd0\x00')

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_authentication_with_user_name_and_password(self):
        asyncio.run(self.authentication_with_user_name_and_password())

    async def authentication_with_user_name_and_password(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Connect with user name and password, and get 0x86 in
            # response.
            reader, writer = await asyncio.open_connection(*address)
            connect = (
                b'\x10\x1c\x00\x04MQTT\x05\xc0\x00\x00\x00\x00\x03su1\x00\x04user'
                b'\x00\x04pass')
            writer.write(connect)
            connack = await reader.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x86\x06\x24\x00\x28\x00\x2a\x00')

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_authentication_method(self):
        asyncio.run(self.authentication_method())

    async def authentication_method(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Connect with authentication method, and get 0x8c in
            # response.
            reader, writer = await asyncio.open_connection(*address)
            connect = (
                b'\x10\x1b\x00\x04MQTT\x05\x00\x00\x00\x0b\x15\x00\x08bad-auth'
                b'\x00\x03su1')
            writer.write(connect)
            connack = await reader.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x8c\x06\x24\x00\x28\x00\x2a\x00')

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_missing_connect_packet(self):
        asyncio.run(self.missing_connect_packet())

    async def missing_connect_packet(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Send a publish packet instead of connect, nothing should
            # be received in response.
            reader, writer = await asyncio.open_connection(*address)
            publish = b'\x30\x0a\x00\x04/a/b\x00apa'
            writer.write(publish)

            with self.assertRaises(asyncio.TimeoutError):
                await asyncio.wait_for(reader.read(1), 0.3)

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_malformed_packet(self):
        asyncio.run(self.malformed_packet())

    async def malformed_packet(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Malformed publish with property 0 should result in a
            # disconnect by the broker.
            reader, writer = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03mal'
            writer.write(connect)
            connack = await reader.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            publish = b'\x30\x0b\x01\x00\x04/a/b\x00apa'
            writer.write(publish)
            disconnect = await reader.readexactly(4)
            self.assertEqual(disconnect, b'\xe0\x02\x81\x00')

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_protocol_error(self):
        asyncio.run(self.protocol_error())

    async def protocol_error(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # It's a protocol error sending ping response to the
            # broker. should result in a disconnect by the broker.
            reader, writer = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03mal'
            writer.write(connect)
            connack = await reader.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            pingresp = b'\xd0\x00'
            writer.write(pingresp)
            disconnect = await reader.readexactly(4)
            self.assertEqual(disconnect, b'\xe0\x02\x82\x00')

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_maximum_packet_size(self):
        asyncio.run(self.maximum_packet_size())

    async def maximum_packet_size(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Connect with maximum packet size set to 50.
            reader, writer = await asyncio.open_connection(*address)
            connect = (
                b'\x10\x15\x00\x04MQTT\x05\x00\x00\x00\x05\x27\x00\x00\x00\x32'
                b'\x00\x03su1')
            writer.write(connect)
            connack = await reader.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Subscribe to a topic.
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer.write(subscribe)
            suback = await reader.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Publish a message of 51 bytes on the subscribed topic,
            # and then one of 50 bytes.
            publish = (
                b'\x30\x31\x00\x04/a/b\x001234567890012345678900123456789001234'
                b'56789')
            self.assertEqual(len(publish), 51)
            writer.write(publish)
            publish = (
                b'\x30\x30\x00\x04/a/b\x001234567890012345678900123456789001234'
                b'5678')
            self.assertEqual(len(publish), 50)
            writer.write(publish)

            # Only the short packet is received.
            publish = await reader.readexactly(50)
            self.assertEqual(
                publish,
                b'\x30\x30\x00\x04/a/b\x001234567890012345678900123456789001234'
                b'5678')

            writer.close()
            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_publish_will(self):
        asyncio.run(self.publish_will())

    async def publish_will(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber. Subscribe to 'foo' and 'a/#'.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x09\x00\x01\x00\x00\x03foo\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')
            subscribe = b'\x82\x09\x00\x01\x00\x00\x03a/#\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Connect with will topic 'foo', message 'bar' and retain
            # False.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = (
                b'\x10\x1a\x00\x04MQTT\x05\x06\x00\x00\x00\x00\x02id\x00\x00'
                b'\x03foo\x00\x03bar')
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Connect with will topic 'a/b', message 'fie' and retain
            # True.
            reader_3, writer_3 = await asyncio.open_connection(*address)
            connect = (
                b'\x10\x1a\x00\x04MQTT\x05\x26\x00\x00\x00\x00\x02ko\x00\x00'
                b'\x03a/b\x00\x03fie')
            writer_3.write(connect)
            connack = await reader_3.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Verify that the will is published when the client is
            # lost.
            writer_2.close()
            publish = await reader_1.readexactly(11)
            self.assertEqual(
                publish,
                b'\x30\x09\x00\x03foo\x00bar')

            # Verify that the will is published when the second client
            # is lost.
            writer_3.close()
            publish = await reader_1.readexactly(11)
            self.assertEqual(
                publish,
                b'\x30\x09\x00\x03a/b\x00fie')

            writer_1.close()

            # Setup the subscriber again with the same subscriptions
            # 'foo' and 'a/#'. Only the retained will a/b should be
            # received.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x09\x00\x01\x00\x00\x03foo\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')
            subscribe = b'\x82\x09\x00\x01\x00\x00\x03a/#\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')
            publish = await reader_1.readexactly(11)
            self.assertEqual(
                publish,
                b'\x30\x09\x00\x03a/b\x00fie')

            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_publish_retained_on_subscribe(self):
        asyncio.run(self.publish_retained_on_subscribe())

    async def publish_retained_on_subscribe(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup a publisher.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03pub'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Publish a topic with retain set.
            publish = b'\x31\x0a\x00\x04/a/b\x00apa'
            writer_1.write(publish)

            # Setup the subscriber.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer_2.write(subscribe)
            suback = await reader_2.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Receive the retained message.
            publish = await reader_2.readexactly(12)
            self.assertEqual(
                publish,
                b'\x30\x0a\x00\x04/a/b\x00apa')

            writer_2.close()

            # Publish the same topic with retain set with no data to
            # remove the retained message from the broker..
            publish = b'\x31\x07\x00\x04/a/b\x00'
            writer_1.write(publish)

            # Setup the subscriber again.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x0a\x00\x01\x00\x00\x04/a/b\x00'
            writer_2.write(subscribe)
            suback = await reader_2.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # No retained message should be received.
            with self.assertRaises(asyncio.TimeoutError):
                await asyncio.wait_for(reader_2.readexactly(1), 0.250)

            writer_2.close()

            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_do_not_publish_will_on_normal_disconnect(self):
        asyncio.run(self.do_not_publish_will_on_normal_disconnect())

    async def do_not_publish_will_on_normal_disconnect(self):
        broker, broker_task = self.create_broker()

        async def tester():
            address = await broker.getsockname()

            # Setup the subscriber. Subscribe to 'foo'.
            reader_1, writer_1 = await asyncio.open_connection(*address)
            connect = b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03su1'
            writer_1.write(connect)
            connack = await reader_1.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')
            subscribe = b'\x82\x09\x00\x01\x00\x00\x03foo\x00'
            writer_1.write(subscribe)
            suback = await reader_1.readexactly(6)
            self.assertEqual(suback, b'\x90\x04\x00\x01\x00\x00')

            # Connect with will topic 'foo' and message 'bar'.
            reader_2, writer_2 = await asyncio.open_connection(*address)
            connect = (
                b'\x10\x1a\x00\x04MQTT\x05\x06\x00\x00\x00\x00\x02id\x00\x00'
                b'\x03foo\x00\x03bar')
            writer_2.write(connect)
            connack = await reader_2.readexactly(11)
            self.assertEqual(
                connack,
                b'\x20\x09\x00\x00\x06\x24\x00\x28\x00\x2a\x00')

            # Verify that the will is not published when the client is
            # normally disconnected.
            disconnect = b'\xe0\x02\x00\x00'
            writer_2.write(disconnect)
            writer_2.close()

            with self.assertRaises(asyncio.TimeoutError):
                await asyncio.wait_for(reader_1.readexactly(1), 0.250)

            writer_1.close()

            broker_task.cancel()

        await asyncio.wait_for(asyncio.gather(broker_task, tester()), 1)

    def test_create_broker_addresses(self):
        asyncio.run(self.create_broker_addresses())

    async def create_broker_addresses(self):
        mqttools.Broker('localhost')
        mqttools.Broker(('localhost', 5))
        mqttools.Broker(('localhost', 10, ssl.create_default_context()))
        mqttools.Broker([
            ('localhost', 5),
            ('localhost', 10, ssl.create_default_context())
        ])


logging.basicConfig(level=logging.DEBUG)


if __name__ == '__main__':
    unittest.main()



import sys
import logging
import asyncio
import unittest
import threading
import binascii
import queue
import socket
from unittest.mock import patch
from io import StringIO

import mqttools


HOST = 'localhost'
PORT = 0


class Broker(threading.Thread):

    EXPECTED_DATA_INDEX = 0
    EXPECTED_DATA_STREAM = []
    ACTUAL_DATA_STREAM = []

    def __init__(self):
        super().__init__()
        self._listener = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self._listener.bind((HOST, PORT))
        self._listener.listen()
        self._client_closed = queue.Queue()

    @property
    def address(self):
        return self._listener.getsockname()

    def wait_for_client_closed(self):
        self._client_closed.get(timeout=1)

    def run(self):
        while True:
            print('Broker: Listening for client...')
            self.serve_client(self._listener.accept()[0])
            self._client_closed.put(True)

    def serve_client(self, client):
        print('Broker: Serving client...')

        while self.EXPECTED_DATA_INDEX < len(self.EXPECTED_DATA_STREAM):
            _, data = self.EXPECTED_DATA_STREAM[self.EXPECTED_DATA_INDEX]

            size = len(data)
            data = client.recv(size)

            if not data:
                break

            self.EXPECTED_DATA_INDEX += 1
            # print(f'Broker: Received: {data}')
            self.ACTUAL_DATA_STREAM.append(('c2s', data))

            while self.EXPECTED_DATA_INDEX < len(self.EXPECTED_DATA_STREAM):
                direction, data = self.EXPECTED_DATA_STREAM[self.EXPECTED_DATA_INDEX]

                if direction != 's2c':
                    break

                self.EXPECTED_DATA_INDEX += 1
                # print(f'Broker: Sending: {data}')
                client.send(data)
                self.ACTUAL_DATA_STREAM.append(('s2c', data))

        client.close()


class MQTToolsTest(unittest.TestCase):

    def setUp(self):
        Broker.EXPECTED_DATA_INDEX = 0
        Broker.EXPECTED_DATA_STREAM = []
        Broker.ACTUAL_DATA_STREAM = []
        Broker.CLOSE_AFTER_INDEX = -1
        self.broker = Broker()
        self.broker.daemon = True
        self.broker.start()
        self.loop = asyncio.new_event_loop()
        asyncio.set_event_loop(self.loop)

    def tearDown(self):
        self.broker.wait_for_client_closed()
        self.loop.close()
        self.assertEqual(Broker.ACTUAL_DATA_STREAM, Broker.EXPECTED_DATA_STREAM)

    def run_until_complete(self, coro):
        return self.loop.run_until_complete(coro)

    def test_start_stop(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 keep_alive_s=0,
                                 topic_alias_maximum=0)
        self.run_until_complete(client.start())
        self.run_until_complete(client.stop())

    def test_subscribe(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # SUBSCRIBE
            ('c2s', b'\x82\n\x00\x01\x00\x00\x04/a/b\x00'),
            # SUBACK
            ('s2c', b'\x90\x04\x00\x01\x00\x00'),
            # SUBSCRIBE
            ('c2s', b'\x82\n\x00\x02\x00\x00\x04/a/c\x00'),
            # SUBACK
            ('s2c', b'\x90\x04\x00\x02\x00\x00'),
            # SUBSCRIBE with invalid topic
            ('c2s', b'\x82\x09\x00\x03\x00\x00\x03/a#\x00'),
            # SUBACK
            ('s2c', b'\x90\x04\x00\x03\x00\xa2'),
            # PUBLISH QoS 0
            ('s2c', b'\x30\x0a\x00\x04/a/b\x00apa'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 keep_alive_s=0,
                                 topic_alias_maximum=0)
        self.run_until_complete(client.start())
        self.run_until_complete(client.subscribe('/a/b'))
        self.run_until_complete(client.subscribe('/a/c'))

        with self.assertRaises(mqttools.SubscribeError) as cm:
            self.run_until_complete(client.subscribe('/a#'))

        self.assertEqual(cm.exception.reason,
                         mqttools.SubackReasonCode.WILDCARD_SUBSCRIPTIONS_NOT_SUPPORTED)
        topic, message = self.run_until_complete(client.messages.get())
        self.assertEqual(topic, '/a/b')
        self.assertEqual(message, b'apa')
        self.run_until_complete(client.stop())

    def test_unsubscribe(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # SUBSCRIBE
            ('c2s', b'\x82\n\x00\x01\x00\x00\x04/a/b\x00'),
            # SUBACK
            ('s2c', b'\x90\x04\x00\x01\x00\x00'),
            # UNSUBSCRIBE
            ('c2s', b'\xa2\x09\x00\x02\x00\x00\x04/a/b'),
            # UNSUBACK
            ('s2c', b'\xb0\x04\x00\x02\x00\x00'),
            # UNSUBSCRIBE from non-subscribed topic
            ('c2s', b'\xa2\x09\x00\x03\x00\x00\x04/a/d'),
            # UNSUBACK
            ('s2c', b'\xb0\x04\x00\x03\x00\x11'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 keep_alive_s=0,
                                 topic_alias_maximum=0)
        self.run_until_complete(client.start())
        self.run_until_complete(client.subscribe('/a/b'))
        self.run_until_complete(client.unsubscribe('/a/b'))

        with self.assertRaises(mqttools.UnsubscribeError) as cm:
            self.run_until_complete(client.unsubscribe('/a/d'))

        self.assertEqual(cm.exception.reason,
                         mqttools.UnsubackReasonCode.NO_SUBSCRIPTION_EXISTED)
        self.run_until_complete(client.stop())

    def test_publish_qos_0(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # PUBLISH
            ('c2s', b'\x30\x0a\x00\x04/a/b\x00apa'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 keep_alive_s=0,
                                 topic_alias_maximum=0)
        self.run_until_complete(client.start())
        client.publish('/a/b', b'apa')
        self.run_until_complete(client.stop())

    def test_command_line_publish_qos_0(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            (
                'c2s',
                b'\x10\x20\x00\x04MQTT\x05\x02\x00<\x03"\x00\n\x00\x10'
                b'mqttools_publish'
            ),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # PUBLISH
            ('c2s', b'\x30\x0a\x00\x04/a/b\x00apa'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        argv = [
            'mqttools',
            'publish',
            '--host', self.broker.address[0],
            '--port', str(self.broker.address[1]),
            '--client-id', 'mqttools_publish',
            '/a/b',
            '617061'
        ]

        stdout = StringIO()

        with patch('sys.stdout', stdout):
            with patch('sys.argv', argv):
                mqttools.main()

        self.assertIn('Published 1 message(s) in', stdout.getvalue())

    def test_command_line_publish_qos_0_generate_message(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            (
                'c2s',
                b'\x10\x20\x00\x04MQTT\x05\x02\x00<\x03"\x00\n\x00\x10'
                b'mqttools_publish'
            ),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # PUBLISH
            ('c2s', b'\x30\x11\x00\x04/a/b\x000\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5\xa5'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        argv = [
            'mqttools',
            'publish',
            '--host', self.broker.address[0],
            '--port', str(self.broker.address[1]),
            '--client-id', 'mqttools_publish',
            '--size', '10',
            '/a/b'
        ]

        stdout = StringIO()

        with patch('sys.stdout', stdout):
            with patch('sys.argv', argv):
                mqttools.main()

        self.assertIn('Published 1 message(s) in', stdout.getvalue())

    def test_command_line_publish_qos_0_generate_short_message(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            (
                'c2s',
                b'\x10\x20\x00\x04MQTT\x05\x02\x00<\x03"\x00\n\x00\x10'
                b'mqttools_publish'
            ),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x000'),
            # PUBLISH
            ('c2s', b'\x30\x08\x00\x04/a/b\x001'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        argv = [
            'mqttools',
            'publish',
            '--host', self.broker.address[0],
            '--port', str(self.broker.address[1]),
            '--client-id', 'mqttools_publish',
            '--count', '11',
            '--size', '1',
            '/a/b'
        ]

        stdout = StringIO()

        with patch('sys.stdout', stdout):
            with patch('sys.argv', argv):
                mqttools.main()

        self.assertIn('Published 11 message(s) in', stdout.getvalue())

    def test_publish_topic_alias(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK with topic alias 5
            ('s2c', b'\x20\x06\x00\x00\x03\x22\x00\x05'),
            # PUBLISH to set alias
            (
                'c2s',
                b'\x30\x2c\x00\x12/test/mqttools/foo\x03\x23\x00\x01'
                b'sets-alias-in-broker'
            ),
            # PUBLISH using alias
            ('c2s', b'\x30\x1a\x00\x00\x03\x23\x00\x01published-with-alias'),
            # PUBLISH without alias
            ('c2s', b'\x30\x24\x00\x12/test/mqttools/fie\x00not-using-alias'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 topic_aliases=[
                                     '/test/mqttools/foo'
                                 ],
                                 topic_alias_maximum=0,
                                 keep_alive_s=0)
        self.run_until_complete(client.start())
        client.publish('/test/mqttools/foo', b'sets-alias-in-broker')
        client.publish('/test/mqttools/foo', b'published-with-alias')
        client.publish('/test/mqttools/fie', b'not-using-alias')
        self.run_until_complete(client.stop())

    def test_use_all_topic_aliases(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK with topic alias 1
            ('s2c', b'\x20\x06\x00\x00\x03\x22\x00\x01'),
            # PUBLISH to set alias
            ('c2s', b'\x30\x0d\x00\x04/foo\x03\x23\x00\x01apa'),
            # PUBLISH, no alias available
            ('c2s', b'\x30\x0a\x00\x04/bar\x00cat'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 topic_aliases=[
                                     '/foo'
                                 ],
                                 topic_alias_maximum=0,
                                 keep_alive_s=0)
        self.run_until_complete(client.start())
        client.publish('/foo', b'apa')
        client.publish('/bar', b'cat')
        self.run_until_complete(client.stop())

    def test_connack_unspecified_error(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT
            ('c2s', b'\x10\x10\x00\x04MQTT\x05\x02\x00\x00\x00\x00\x03bar'),
            # CONNACK with unspecified error
            ('s2c', b'\x20\x03\x00\x80\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 topic_alias_maximum=0,
                                 connect_delays=[],
                                 keep_alive_s=0)

        with self.assertRaises(mqttools.ConnectError) as cm:
            self.run_until_complete(client.start())

        self.assertEqual(str(cm.exception), 'UNSPECIFIED_ERROR(128)')

    def test_receive_topic_alias(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT with topic alias 5
            (
                'c2s',
                b'\x10\x13\x00\x04MQTT\x05\x02\x00\x00\x03\x22\x00\x05\x00\x03bar'
            ),
            # CONNACK
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # SUBSCRIBE
            ('c2s', b'\x82\x18\x00\x01\x00\x00\x12/test/mqttools/foo\x00'),
            # SUBACK
            ('s2c', b'\x90\x04\x00\x01\x00\x00'),
            # PUBLISH using an unknown alias 1
            (
                's2c',
                b'\x30\x22\x00\x00\x03\x23\x00\x01published-with-unknown-alias'
            ),
            # PUBLISH using alias an invalid alias 6
            (
                's2c',
                b'\x30\x34\x00\x12/test/mqttools/foo\x03\x23\x00\x06'
                b'sets-invalid-alias-in-client'
            ),

            # PUBLISH to set alias
            (
                's2c',
                b'\x30\x2c\x00\x12/test/mqttools/foo\x03\x23\x00\x01'
                b'sets-alias-in-client'
            ),
            # PUBLISH using alias
            ('s2c', b'\x30\x1a\x00\x00\x03\x23\x00\x01published-with-alias'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 topic_alias_maximum=5,
                                 keep_alive_s=0)
        self.run_until_complete(client.start())
        self.run_until_complete(client.subscribe('/test/mqttools/foo'))
        topic, message = self.run_until_complete(client.messages.get())
        self.assertEqual(topic, '/test/mqttools/foo')
        self.assertEqual(message, b'sets-alias-in-client')
        topic, message = self.run_until_complete(client.messages.get())
        self.assertEqual(topic, '/test/mqttools/foo')
        self.assertEqual(message, b'published-with-alias')
        self.run_until_complete(client.stop())

    def test_resume_session(self):
        Broker.EXPECTED_DATA_STREAM = [
            # CONNECT with clean session 0 (to resume) and session
            # expiry interval 120.
            (
                'c2s',
                b'\x10\x15\x00\x04MQTT\x05\x00\x00\x00\x05\x11\x00\x00\x00\x78'
                b'\x00\x03bar'
            ),
            # CONNACK with no session present
            ('s2c', b'\x20\x03\x00\x00\x00'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00'),
            # CONNECT with clean session 0 (to resume) and session
            # expiry interval 120.
            (
                'c2s',
                b'\x10\x15\x00\x04MQTT\x05\x00\x00\x00\x05\x11\x00\x00\x00\x78'
                b'\x00\x03bar'
            ),
            # CONNACK with session present
            ('s2c', b'\x20\x03\x01\x00\x00'),
            # DISCONNECT
            ('c2s', b'\xe0\x02\x00\x00')
        ]

        client = mqttools.Client(*self.broker.address,
                                 'bar',
                                 session_expiry_interval=120,
                                 topic_alias_maximum=0,
                                 connect_delays=[],
                                 keep_alive_s=0)

        with self.assertRaises(mqttools.SessionResumeError):
            self.run_until_complete(client.start(resume_session=True))

        self.run_until_complete(client.stop())
        self.broker.wait_for_client_closed()
        self.run_until_complete(client.start(resume_session=True))
        self.run_until_complete(client.stop())


logging.basicConfig(level=logging.DEBUG)


if __name__ == '__main__':
    unittest.main()



import logging
import asyncio
import unittest

import mqttools


async def broker_main(listener):
    async with listener:
        try:
            await listener.serve_forever()
        except asyncio.CancelledError:
            pass


class ClientTest(unittest.TestCase):

    def test_connack_timeout(self):
        asyncio.run(self.connack_timeout())

    async def connack_timeout(self):
        def on_client_connected(reader, writer):
            pass

        listener = await asyncio.start_server(on_client_connected, 'localhost', 0)

        async def client_main():
            client = mqttools.Client(*listener.sockets[0].getsockname(),
                                     'connack',
                                     response_timeout=0.1,
                                     connect_delays=[],
                                     topic_alias_maximum=0)

            with self.assertRaises(mqttools.TimeoutError):
                await client.start()

            listener.close()

        await asyncio.wait_for(
            asyncio.gather(broker_main(listener), client_main()), 1)

    def test_subscribe_timeout(self):
        asyncio.run(self.subscribe_timeout())

    async def subscribe_timeout(self):
        def on_client_connected(reader, writer):
            # CONNACK
            writer.write(b'\x20\x03\x00\x00\x00')

        listener = await asyncio.start_server(on_client_connected, 'localhost', 0)

        async def client_main():
            client = mqttools.Client(*listener.sockets[0].getsockname(),
                                     'suback',
                                     response_timeout=0.1,
                                     topic_alias_maximum=0)
            await client.start()

            with self.assertRaises(mqttools.TimeoutError):
                await client.subscribe('/foo')

            listener.close()

        await asyncio.wait_for(
            asyncio.gather(broker_main(listener), client_main()), 1)

    def test_unsubscribe_timeout(self):
        asyncio.run(self.unsubscribe_timeout())

    async def unsubscribe_timeout(self):
        def on_client_connected(reader, writer):
            # CONNACK
            writer.write(b'\x20\x03\x00\x00\x00')

        listener = await asyncio.start_server(on_client_connected, 'localhost', 0)

        async def client_main():
            client = mqttools.Client(*listener.sockets[0].getsockname(),
                                     'unsuback',
                                     response_timeout=0.1,
                                     topic_alias_maximum=0)
            await client.start()

            with self.assertRaises(mqttools.TimeoutError):
                await client.unsubscribe('/foo')

            listener.close()

        await asyncio.wait_for(
            asyncio.gather(broker_main(listener), client_main()), 1)

    def test_client_id(self):
        client = mqttools.Client('localhost', 0)
        self.assertEqual(client.client_id[:9], 'mqttools-')



logging.basicConfig(level=logging.DEBUG)


if __name__ == '__main__':
    unittest.main()



import asyncio
import ssl

from ..broker import Broker


def _do_broker(args):
    addresses = [(args.host, args.port)]

    if all([args.cafile, args.certfile, args.keyfile]):
        context = ssl.create_default_context(ssl.Purpose.CLIENT_AUTH,
                                             cafile=args.cafile)
        context.check_hostname = not args.no_check_hostname
        context.load_cert_chain(certfile=args.certfile, keyfile=args.keyfile)
        addresses.append((args.host, args.secure_port, context))

    broker = Broker(addresses)
    asyncio.run(broker.serve_forever())


def add_subparser(subparsers):
    subparser = subparsers.add_parser('broker',
                                      description='A simple broker.')
    subparser.add_argument('--host',
                           default='localhost',
                           help="Broker host (default: %(default)s).")
    subparser.add_argument('--port',
                           type=int,
                           default=1883,
                           help='Broker port (default: %(default)s).')
    subparser.add_argument('--secure-port',
                           type=int,
                           default=8883,
                           help='Secure broker port (default: %(default)s).')
    subparser.add_argument(
        '--cafile',
        default='',
        help='CA file.')
    subparser.add_argument(
        '--certfile',
        default='',
        help='MQTT broker certificate file.')
    subparser.add_argument(
        '--keyfile',
        default='',
        help='MQTT broker key file.')
    subparser.add_argument(
        '--no-check-hostname',
        action='store_true',
        help='Do not check certificate hostname.')
    subparser.set_defaults(func=_do_broker)



#!/usr/bin/env python3

import re
import setuptools
from setuptools import find_packages


def find_version():
    return re.search(r"^__version__ = '(.*)'$",
                     open('mqttools/version.py', 'r').read(),
                     re.MULTILINE).group(1)


setuptools.setup(
    name='mqttools',
    version=find_version(),
    description='MQTT tools.',
    long_description=open('README.rst', 'r').read(),
    author='Erik Moqvist',
    author_email='erik.moqvist@gmail.com',
    license='MIT',
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],
    keywords=['mqtt'],
    url='https://github.com/eerimoq/mqttools',
    packages=find_packages(exclude=['tests']),
    install_requires=[
        'bitstruct',
        'humanfriendly',
        'windows-curses;platform_system=="Windows"'
    ],
    test_suite="tests",
    entry_points={
        'console_scripts': ['mqttools=mqttools.__init__:main']
    })
